@@ -1,19 +1,18 @@
 /* dbstubs.c
  * Stubs for the database functions.
  * After all, most people will compile this without database access.
- * Copyright (c) Karl Dahlke, 2008
  * This file is part of the edbrowse project, released under GPL.
  */
 
 #include "eb.h"
 
-eb_bool sqlPresent = eb_false;
+bool sqlPresent = false;
 
-eb_bool sqlReadRows(const char *filename, char **bufptr)
+bool sqlReadRows(const char *filename, char **bufptr)
 {
 	setError(MSG_DBNotCompiled);
-	*bufptr = EMPTYSTRING;
-	return eb_false;
+	*bufptr = emptyString;
+	return false;
 }				/* sqlReadRows */
 
 void dbClose(void)
@@ -28,30 +27,34 @@ void showForeign(void)
 {
 }				/* showForeign */
 
-eb_bool showTables(void)
+bool showTables(void)
 {
+	return false;
 }				/* showTables */
 
-eb_bool sqlDelRows(int start, int end)
+bool sqlDelRows(int start, int end)
 {
+	return false;
 }				/* sqlDelRows */
 
-eb_bool sqlUpdateRow(pst source, int slen, pst dest, int dlen)
+bool sqlUpdateRow(pst source, int slen, pst dest, int dlen)
 {
+	return false;
 }				/* sqlUpdateRow */
 
-eb_bool sqlAddRows(int ln)
+bool sqlAddRows(int ln)
 {
+	return false;
 }				/* sqlAddRows */
 
-eb_bool ebConnect(void)
+bool ebConnect(void)
 {
 	setError(MSG_DBNotCompiled);
-	return eb_false;
+	return false;
 }				/* ebConnect */
 
 int goSelect(int *startLine, char **rbuf)
 {
-	*rbuf = EMPTYSTRING;
+	*rbuf = emptyString;
 	return -1;
 }				/* goSelect */
