@@ -21,6 +21,7 @@ nodeFunction traverse_callback;
 /* possible callback functions in this file */
 static void prerenderNode(struct htmlTag *node, bool opentag);
 static void jsNode(struct htmlTag *node, bool opentag);
+static void pushAttributes(const struct htmlTag *t);
 
 static void processStyles(jsobjtype so, const char *stylestring);
 
@@ -50,7 +51,6 @@ void traverseAll(int start)
 	int i;
 
 	treeOverflow = false;
-
 	for (i = start; i < cw->numTags; ++i) {
 		t = tagList[i];
 		t->visited = false;
@@ -58,7 +58,7 @@ void traverseAll(int start)
 
 	for (i = start; i < cw->numTags; ++i) {
 		t = tagList[i];
-		if (!t->parent && !t->slash && t->step < 100)
+		if (!t->parent && !t->slash && !t->dead)
 			traverseNode(t);
 	}
 
@@ -69,7 +69,7 @@ void traverseAll(int start)
 static int nopt;		/* number of options */
 /* None of these tags nest, so it is reasonable to talk about
  * the current open tag. */
-static struct htmlTag *currentForm, *currentSel, *currentOpt;
+static struct htmlTag *currentForm, *currentSel, *currentOpt, *currentStyle;
 static struct htmlTag *currentTitle, *currentScript, *currentTA;
 static struct htmlTag *currentA;
 static char *radioCheck;
@@ -124,15 +124,21 @@ static void makeButton(void)
 
 struct htmlTag *findOpenTag(struct htmlTag *t, int action)
 {
-	while (t = t->parent)
+	int count = 0;
+	while ((t = t->parent)) {
 		if (t->action == action)
 			return t;
+		if (++count == 10000) {	// tree shouldn't be this deep
+			debugPrint(1, "infinite loop in findOpenTag()");
+			break;
+		}
+	}
 	return 0;
 }				/* findOpenTag */
 
 struct htmlTag *findOpenList(struct htmlTag *t)
 {
-	while (t = t->parent)
+	while ((t = t->parent))
 		if (t->action == TAGACT_OL || t->action == TAGACT_UL)
 			return t;
 	return 0;
@@ -314,17 +320,16 @@ void formControl(struct htmlTag *t, bool namecheck)
 	}
 	if (cform)
 		t->controller = cform;
-	else if (itype != INP_BUTTON)
+	else if (itype != INP_BUTTON && itype != INP_SUBMIT && !htmlGenerated)
 		debugPrint(3, "%s is not part of a fill-out form",
 			   t->info->desc);
-	if (namecheck && !myname)
+	if (namecheck && !myname && !htmlGenerated)
 		debugPrint(3, "%s does not have a name", t->info->desc);
 }				/* formControl */
 
-static const char *const inp_types[] = {
+const char *const inp_types[] = {
 	"reset", "button", "image", "submit",
-	"hidden",
-	"text", "password", "number", "file",
+	"hidden", "text", "file",
 	"select", "textarea", "radio", "checkbox",
 	0
 };
@@ -336,10 +341,11 @@ they are equivalent to text. Just here to suppress warnings.
 List taken from https://www.tutorialspoint.com/html/html_input_tag.htm
 *********************************************************************/
 
-static const char *const inp_others[] = {
-	"date", "datetime", "datetime-local",
-	"month", "week", "time",
-	"email", "range", "search", "tel", "url", 0
+const char *const inp_others[] = {
+	"no_minor", "date", "datetime", "datetime-local",
+	"month", "week", "time", "email", "range",
+	"search", "tel", "url", "number", "password",
+	0
 };
 
 /* helper function for input tag */
@@ -351,12 +357,17 @@ void htmlInputHelper(struct htmlTag *t)
 	const char *s = attribVal(t, "type");
 	if (stringEqual(t->info->name, "button")) {
 		n = INP_BUTTON;
-	} else if (s) {
+	}
+	if (s) {
 		n = stringInListCI(inp_types, s);
 		if (n < 0) {
 			n = stringInListCI(inp_others, s);
 			if (n < 0)
 				debugPrint(3, "unrecognized input type %s", s);
+			else
+				t->itype_minor = n;
+			if (n == INP_PW)
+				t->masked = true;
 			n = INP_TEXT;
 		}
 	}
@@ -432,7 +443,7 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 	int j;
 	int action = t->action;
 	const char *a;		/* usually an attribute */
-	struct htmlTag *u;
+	struct htmlTag *cdt;
 
 	debugPrint(6, "prend %c%s %d%s",
 		   (opentag ? ' ' : '/'), t->info->name,
@@ -444,6 +455,12 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 		t->step = 1;
 
 	switch (action) {
+	case TAGACT_NOSCRIPT:
+// If javascript is enabled kill everything under noscript
+		if (allowJS && !opentag)
+			underKill(t);
+		break;
+
 	case TAGACT_TEXT:
 		if (!opentag || !t->textval)
 			break;
@@ -464,6 +481,12 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 			break;
 		}
 
+		if (currentStyle) {
+			currentStyle->textval = cloneString(t->textval);
+			t->deleted = true;
+			break;
+		}
+
 		if (currentScript) {
 			currentScript->textval = cloneString(t->textval);
 			t->deleted = true;
@@ -534,7 +557,7 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 					debugPrint(3,
 						   "unrecognized enctype, plese use multipart/form-data or application/x-www-form-urlencoded");
 			}
-			if (a = t->href) {
+			if ((a = t->href)) {
 				const char *prot = getProtURL(a);
 				if (prot) {
 					if (stringEqualCI(prot, "mailto"))
@@ -608,6 +631,15 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 		t->textval = emptyString;
 		break;
 
+	case TAGACT_STYLE:
+		if (!opentag) {
+			currentStyle = 0;
+			break;
+		}
+		currentStyle = t;
+		t->textval = emptyString;
+		break;
+
 	case TAGACT_SELECT:
 		if (opentag) {
 			currentSel = t;
@@ -633,8 +665,10 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 /* like the other value fields, it can't be null */
 				t->rvalue = t->value = emptyString;
 			}
-			j = sideBuffer(0, t->value, -1, 0);
-			t->lic = j;
+			if (whichproc == 'e') {
+				j = sideBuffer(0, t->value, -1, 0);
+				t->lic = j;
+			}
 			currentTA = 0;
 		}
 		break;
@@ -665,7 +699,7 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 	case TAGACT_SPAN:
 		if (!opentag)
 			break;
-		if (!(a = t->classname))
+		if (!(a = t->class))
 			break;
 		if (stringEqualCI(a, "sup"))
 			action = TAGACT_SUP;
@@ -688,13 +722,11 @@ static void prerenderNode(struct htmlTag *t, bool opentag)
 	case TAGACT_FRAME:
 		if (opentag)
 			break;
-/* since this browser supports frames, we don't show the text inside */
-		for (u = t->firstchild; u; u = u->sibling) {
-			u->parent = 0;
-			u->deleted = true;
-			u->step = 100;
-		}
-		t->firstchild = 0;
+// If somebody wrote <frame><p><a></frame>, those tags should be excised,
+		underKill(t);
+		cdt = newTag("document");
+		t->firstchild = cdt;
+		cdt->parent = t;
 		break;
 
 	}			/* switch */
@@ -709,6 +741,7 @@ void prerender(int start)
 
 	currentForm = currentSel = currentOpt = NULL;
 	currentTitle = currentScript = currentTA = NULL;
+	currentStyle = NULL;
 	nzFree(radioCheck);
 	radioCheck = 0;
 	traverse_callback = prerenderNode;
@@ -730,15 +763,45 @@ jsobjtype instantiate_url(jsobjtype parent, const char *name, const char *url)
 
 static void handlerSet(jsobjtype ev, const char *name, const char *code)
 {
-	enum ej_proptype hasform = has_property(ev, "form");
 	char *newcode = allocMem(strlen(code) + 60);
+	strcpy(newcode, "(function(){");
+
+/*********************************************************************
+I read somewhere, long ago, that handlers are suppose to run in the context of
+the document object and sometimes the form that houses them.
+So I should put with(document) and sometimes with(form) around the code.
+For onclick, the code just runs, but for onsubmit the code is suppose to
+return true or false.
+Mozilla had no trouble compiling and running  return true;  at top level.
+Duktape won't do that. Return has to be in a function.
+I even tried DUK_COMPILE_FUNCTION, no dice.
+So I tried wrapping the code in
+(function() { code })();
+Then it doesn't matter if the code is just expression, or return expression.
+But should the with clauses be inside or outside the function?
+Also, the this binding was lost,
+which is absolutely necessary.
+Sami taught me how to get the binding back, via the .bind method,
+which is pretty dog gone cool.
+But what about the with clauses?
+Well so far I haven't found an example that needs with(document) around it,
+so I'm not going to worry about that right now.
+
 	strcpy(newcode, "with(document) { ");
+	enum ej_proptype hasform = typeof_property(ev, "form");
 	if (hasform)
 		strcat(newcode, "with(this.form) { ");
+*********************************************************************/
+
 	strcat(newcode, code);
+
+/*********************************************************************
 	if (hasform)
 		strcat(newcode, " }");
 	strcat(newcode, " }");
+*********************************************************************/
+
+	strcat(newcode, " }.bind(this))()");
 	set_property_function(ev, name, newcode);
 	nzFree(newcode);
 }				/* handlerSet */
@@ -776,7 +839,7 @@ static const char *fakePropName(void)
 {
 	static int idx = 0;
 	++idx;
-	sprintf(fakePropLast, "gc$$%d", idx);
+	sprintf(fakePropLast, "gc$%c%d", whichproc, idx);
 	return fakePropLast;
 }				/*fakePropName */
 
@@ -784,6 +847,7 @@ static jsobjtype establish_js_option(jsobjtype obj, int idx)
 {
 	jsobjtype oa;		/* option array */
 	jsobjtype oo;		/* option object */
+	jsobjtype so;		// style object
 	jsobjtype fo;		/* form object */
 
 	if ((oa = get_property_object(obj, "options")) == NULL)
@@ -791,13 +855,16 @@ static jsobjtype establish_js_option(jsobjtype obj, int idx)
 	if ((oo = instantiate_array_element(oa, idx, "Option")) == NULL)
 		return NULL;
 
+	set_property_object(oo, "parentNode", oa);
+
 /* option.form = select.form */
 	fo = get_property_object(obj, "form");
 	if (fo)
 		set_property_object(oo, "form", fo);
 	instantiate_array(oo, "childNodes");
 	instantiate_array(oo, "attributes");
-	instantiate(oo, "style", 0);
+	so = instantiate(oo, "style", "CSSStyleDeclaration");
+	set_property_object(so, "element", oo);
 
 	return oo;
 }				/* establish_js_option */
@@ -824,55 +891,35 @@ static void domLink(struct htmlTag *t, const char *classname,	/* instantiate thi
 	jsobjtype master;
 	jsobjtype alist = 0;
 	jsobjtype io = 0;	/* input object */
-	unsigned length;
+	int length;
 	bool dupname = false;
 /* some strings from the html tag */
 	const char *symname = t->name;
 	const char *idname = t->id;
 	const char *membername = 0;	/* usually symname */
 	const char *href_url = t->href;
-	const char *htmlclass = t->classname;
 	const char *stylestring = attribVal(t, "style");
 	jsobjtype so = 0;	/* obj.style */
 
 	debugPrint(5, "domLink %s.%d name %s",
 		   classname, radiosel, (symname ? symname : emptyString));
 
-	if (symname && has_property(owner, symname)) {
+	if (symname && typeof_property(owner, symname)) {
 /*********************************************************************
 This could be a duplicate name.
 Yes, that really happens.
 Link to the first tag having this name,
-and link the second tag under a fake name, so gc won't throw it away.
+and link the second tag under a fake name so gc won't throw it away.
 Or - it could be a duplicate name because multiple radio buttons
 all share the same name.
-The first time, we create the array,
+The first time we create the array,
 and thereafter we just link under that array.
 Or - and this really does happen -
 an input tag could have the name action, colliding with form.action.
 I have no idea what to do here.
-I will assume the tag displaces the action.
-That means javascript cannot change the action of the form,
-which it rarely does anyways.
-When it refers to form.action, that will be the input tag.
-I'll check for that one first.
-Yeah, it makes my head spin too.
+radiosel is 1 for radio buttons and 2 for select.
 *********************************************************************/
 
-		if (stringEqual(symname, "action")) {
-			jsobjtype ao;	/* action object */
-			ao = get_property_object(owner, symname);
-			if (ao == NULL)
-				return;
-/* actioncrash tells me if we've already had this collision */
-			if (!has_property(ao, "actioncrash")) {
-				delete_property(owner, symname);
-/* advance, as though this were not found */
-				goto afterfound;
-			}
-		}
-
-/* radiosel is 1 for radio buttons and 2 for select */
 		if (radiosel == 1) {
 /* name present and radio buttons, name should be the array of buttons */
 			io = get_property_object(owner, symname);
@@ -884,11 +931,9 @@ Yeah, it makes my head spin too.
 		}
 	}
 
-afterfound:
 /* The input object is nonzero if&only if the input is a radio button,
  * and not the first button in the set, thus it isce the array containing
  * these buttons. */
-
 	if (io == NULL) {
 /*********************************************************************
 Ok, the above condition does not hold.
@@ -898,15 +943,20 @@ or id= if there is no name=, or a fake name just to protect it from gc.
 *********************************************************************/
 
 		if (!symname && idname) {
-/* id= must not displace submit, reset, or action.
- * Example www.startpage.com, where id=submit */
-			if (!stringEqual(idname, "submit") &&
-			    !stringEqual(idname, "reset") &&
-			    !stringEqual(idname, "action"))
-				membername = idname;
+			membername = idname;
 		} else if (symname && !dupname) {
 			membername = symname;
 		}
+/* id= or name= must not displace submit, reset, or action in form.
+ * Example www.startpage.com, where id=submit.
+ * nor should it collide with another attribute, such as document.cookie and
+ * <div ID=cookie> in www.orange.com.
+ * This call checks for the name in the object and its prototype. */
+		if (has_property(owner, membername)) {
+			debugPrint(3, "membername overload %s.%s", classname,
+				   membername);
+			membername = NULL;
+		}
 		if (!membername)
 			membername = fakePropName();
 
@@ -936,30 +986,33 @@ or id= if there is no name=, or a fake name just to protect it from gc.
 /* not an array; needs the childNodes array beneath it for the children */
 			instantiate_array(io, "childNodes");
 
-/* deal with the 'styles' here */
-/* object will get 'style' regardless of whether there is
+/* in the special case of form, also need an array of elements */
+			if (t->action == TAGACT_FORM)
+				instantiate_array(io, "elements");
+		}
+
+/* deal with the 'styles' here.
+object will get 'style' regardless of whether there is
 anything to put under it, just like it gets childNodes whether
 or not there are any.  After that, there is a conditional step.
 If this node contains style='' of one or more name-value pairs,
-call out to process those and add them to the object */
-			so = instantiate(io, "style", 0);
+call out to process those and add them to the object.
+Don't do any of this if the tag is itself <style>. */
+		if (t->action != TAGACT_STYLE) {
+			so = instantiate(io, "style", "CSSStyleDeclaration");
+			set_property_object(so, "element", io);
 /* now if there are any style pairs to unpack,
  processStyles can rely on obj.style existing */
 			if (stylestring)
 				processStyles(so, stylestring);
+		}
 
 /* Other attributes that are expected by pages, even if they
  * aren't populated at domLink-time */
-			set_property_string(io, "className", "");
-			set_property_string(io, "class", "");
-			set_property_string(io, "nodeValue", "");
-			instantiate_array(io, "attributes");
-			set_property_object(io, "ownerDocument", cf->docobj);
-
-/* in the special case of form, also need an array of elements */
-			if (stringEqual(classname, "Form"))
-				instantiate_array(io, "elements");
-		}
+		set_property_string(io, "class", "");
+		set_property_string(io, "nodeValue", "");
+		instantiate_array(io, "attributes");
+		set_property_object(io, "ownerDocument", cf->docobj);
 
 		if (membername == symname) {
 /* link to document.all */
@@ -979,9 +1032,11 @@ call out to process those and add them to the object */
 			if (length < 0)
 				return;
 			set_array_element_object(alist, length, io);
-			if (symname && !dupname)
+			if (symname && !dupname
+			    && !has_property(alist, symname))
 				set_property_object(alist, symname, io);
-			if (idname && membername != idname)
+			if (idname && symname != idname
+			    && !has_property(alist, idname))
 				set_property_object(alist, idname, io);
 		}		/* list indicated */
 	}
@@ -1004,49 +1059,34 @@ call out to process those and add them to the object */
 		set_property_string(io, "name", symname);
 
 	if (idname) {
-/* io.id becomes idname, and idMaster.idname becomes io
- * In case of forms, v.id should remain undefined.  So we can have
- * a form field named "id". */
-		if (!stringEqual(classname, "Form"))
-			set_property_string(io, "id", idname);
+/* io.id becomes idname, and idMaster.idname becomes io */
+		set_property_string(io, "id", idname);
 		master = get_property_object(cf->docobj, "idMaster");
 		set_property_object(master, idname, io);
 	}
 
 	if (href && href_url)
-		instantiate_url(io, href, href_url);
+// This use to be instantiate_url, but with the new side effects
+// on Anchor, Image, etc, we can just set the string.
+		set_property_string(io, href, href_url);
 
-	if (stringEqual(classname, "Element")) {
+	if (t->action == TAGACT_INPUT) {
 /* link back to the form that owns the element */
 		set_property_object(io, "form", owner);
 	}
 
-	if (htmlclass) {
-		set_property_string(io, "className", htmlclass);
-		set_property_string(io, "class", htmlclass);
-	}
-
-	t->jv = io;
+	connectTagObject(t, io);
 
 	set_property_string(io, "nodeName", t->info->name);
 /* documentElement is now set in the "Body" case because the 
 "Html" does not appear ever to be encountered */
 
-	if (stringEqual(classname, "Body")) {
-/* here are a few attributes that come in with the body */
+	if (t->action == TAGACT_BODY) {
 		set_property_object(cf->docobj, "body", io);
-		set_property_number(io, "clientHeight", 768);
-		set_property_number(io, "clientWidth", 1024);
-		set_property_number(io, "offsetHeight", 768);
-		set_property_number(io, "offsetWidth", 1024);
-		set_property_number(io, "scrollHeight", 768);
-		set_property_number(io, "scrollWidth", 1024);
-		set_property_number(io, "scrollTop", 0);
-		set_property_number(io, "scrollLeft", 0);
 		set_property_object(cf->docobj, "documentElement", io);
 	}
 
-	if (stringEqual(classname, "Head")) {
+	if (t->action == TAGACT_HEAD) {
 		set_property_object(cf->docobj, "head", io);
 	}
 
@@ -1062,7 +1102,6 @@ static void formControlJS(struct htmlTag *t)
 	int itype = t->itype;
 	int isradio = itype == INP_RADIO;
 	int isselect = (itype == INP_SELECT) * 2;
-	char *myname = (t->name ? t->name : t->id);
 	const struct htmlTag *form = t->controller;
 
 	if (form && form->jv)
@@ -1114,7 +1153,7 @@ static void optionJS(struct htmlTag *t)
 	if (!sel->jv)
 		return;
 
-	t->jv = establish_js_option(sel->jv, t->lic);
+	connectTagObject(t, establish_js_option(sel->jv, t->lic));
 	set_property_string(t->jv, "text", t->textval);
 	set_property_string(t->jv, "value", t->value);
 	set_property_string(t->jv, "nodeName", "option");
@@ -1127,15 +1166,66 @@ static void optionJS(struct htmlTag *t)
 	}
 }				/* optionJS */
 
+static void link_css(struct htmlTag *t)
+{
+	const char *a = attribVal(t, "type");
+	if (a)
+		set_property_string(t->jv, "type", a);
+	if (!t->href)
+		return;
+	if (!a || !stringEqualCI(a, "text/css"))
+		return;
+// Fetch the css file so we can apply its attributes.
+	a = NULL;
+	if (browseLocal && !isURL(t->href)) {
+		debugPrint(3, "css source %s", t->href);
+		if (!fileIntoMemory(t->href, &serverData, &serverDataLen)) {
+			if (debugLevel >= 1)
+				i_printf(MSG_GetLocalCSS, errorMsg);
+		} else {
+			a = force_utf8(serverData, serverDataLen);
+			if (!a)
+				a = serverData;
+			else
+				nzFree(serverData);
+			serverData = NULL;
+			serverDataLen = 0;
+		}
+	} else {
+		debugPrint(3, "css source %s", t->href);
+		if (httpConnect(t->href, false, false, true, 0, 0, 0)) {
+			if (ht_code == 200) {
+				a = force_utf8(serverData, serverDataLen);
+				if (!a)
+					a = serverData;
+				else
+					nzFree(serverData);
+			} else {
+				nzFree(serverData);
+				if (debugLevel >= 3)
+					i_printf(MSG_GetCSS, t->href, ht_code);
+			}
+			serverData = NULL;
+			serverDataLen = 0;
+		} else {
+			if (debugLevel >= 3)
+				i_printf(MSG_GetCSS2, errorMsg);
+		}
+	}
+	if (a)
+		set_property_string(t->jv, "data", a);
+	cnzFree(a);
+}				/* link_css */
+
 static jsobjtype innerParent;
 
 static void jsNode(struct htmlTag *t, bool opentag)
 {
-	int itype;		/* input type */
 	const struct tagInfo *ti = t->info;
 	int action = t->action;
 	const struct htmlTag *above;
 	const char *a;
+	bool linked_in;
 
 /* all the js variables are on the open tag */
 	if (!opentag)
@@ -1154,25 +1244,19 @@ Needless to say that's not good!
 		return;
 
 	debugPrint(6, "decorate %s %d", t->info->name, t->seqno);
+	fakePropLast[0] = 0;
 
 	switch (action) {
-		jsobjtype cd;	/* contentDocument */
-		jsobjtype cdbody;	/* contentDocument.body */
 
 	case TAGACT_TEXT:
-		t->jv = instantiate(cf->docobj, fakePropName(), "TextNode");
+		connectTagObject(t,
+				 instantiate(cf->docobj, fakePropName(),
+					     "TextNode"));
 		if (t->jv) {
 			const char *w = t->textval;
 			if (!w)
 				w = emptyString;
 			set_property_string(t->jv, "data", w);
-			set_property_string(t->jv, "nodeName", "text");
-			set_property_number(t->jv, "nodeType", 3);
-/* A text node chould never have children, and does not need childNodes array,
- * but there is improper html out there <text> <stuff>
- * which has to put stuff under the text node, so against this
- * unlikely occurence, I have to create the array. */
-			instantiate_array(t->jv, "childNodes");
 		}
 		break;
 
@@ -1183,6 +1267,17 @@ Needless to say that's not good!
 		set_property_number(t->jv, "nodeType", 1);
 		break;
 
+	case TAGACT_STYLE:
+		domLink(t, "CSSStyleDeclaration", 0, "styles", cf->docobj, 0);
+		a = attribVal(t, "type");
+		if (!a)
+			a = emptyString;
+		set_property_string(t->jv, "type", a);
+		set_property_number(t->jv, "nodeType", 1);
+		if (t->textval && t->textval[0])
+			set_property_string(t->jv, "data", t->textval);
+		break;
+
 	case TAGACT_SCRIPT:
 		domLink(t, "Script", "src", "scripts", cf->docobj, 0);
 		a = attribVal(t, "type");
@@ -1205,6 +1300,7 @@ Needless to say that's not good!
 		}
 		set_property_number(t->jv, "nodeType", 1);
 		break;
+
 	case TAGACT_FORM:
 		domLink(t, "Form", "action", "forms", cf->docobj, 0);
 		set_onhandlers(t);
@@ -1240,6 +1336,7 @@ Needless to say that's not good!
 		domLink(t, "Body", 0, "bodies", cf->docobj, 0);
 		set_onhandlers(t);
 		set_property_number(t->jv, "nodeType", 1);
+		cf->bodytag = t;
 		break;
 
 	case TAGACT_OL:
@@ -1300,15 +1397,14 @@ Needless to say that's not good!
 		break;
 
 	case TAGACT_FRAME:
+// about:blank means a blank frame with no sourcefile.
+		if (stringEqual(t->href, "about:blank")) {
+			nzFree(t->href);
+			t->href = 0;
+		}
 		domLink(t, "Frame", "src", "frames", cf->winobj, 0);
+		set_onhandlers(t);
 		set_property_number(t->jv, "nodeType", 1);
-/* frames have contentDocument below, even though it is not populated
- * with the frame's dom as of this version of edbrowse. */
-		cd = instantiate(t->jv, "contentDocument", "Document");
-		cdbody = instantiate(cd, "body", "Body");
-		instantiate(cdbody, "style", 0);
-		instantiate_url(cd, "location", t->href);
-/* perhaps other stuff that a frame document always needs */
 		break;
 
 	case TAGACT_IMAGE:
@@ -1321,40 +1417,119 @@ Needless to say that's not good!
 		set_property_number(t->jv, "nodeType", 1);
 		break;
 
-	case TAGACT_TITLE:
-		if (cw->ft)
-			set_property_string(cf->docobj, "title", cw->ft);
+	case TAGACT_HEADER:
+		domLink(t, "Header", 0, "headers", cf->docobj, 0);
 		set_property_number(t->jv, "nodeType", 1);
 		break;
 
+	case TAGACT_FOOTER:
+		domLink(t, "Footer", 0, "footers", cf->docobj, 0);
+		set_property_number(t->jv, "nodeType", 1);
+		break;
+
+	case TAGACT_TITLE:
+		if (cw->ft)
+			set_property_string(cf->docobj, "title", cw->ft);
+// Title is not a node, more like an attribute of the page.
+		break;
+
+	case TAGACT_LINK:
+		domLink(t, "Link", "href", "links", cf->docobj, 0);
+		set_property_number(t->jv, "nodeType", 1);
+		link_css(t);
+		break;
+
+	default:
+// Don't know what this tag is, or it's not semantically important,
+// so just call it an html element.
+		domLink(t, "Element", 0, 0, cf->docobj, 0);
+		if (t->action == TAGACT_BASE && t->href)
+			instantiate_url(t->jv, "href", t->href);
+		break;
 	}			/* switch */
 
 	if (!t->jv)
 		return;		/* nothing else to do */
 
 /* js tree mirrors the dom tree. */
-	if (t->parent && t->parent->jv)
-		run_function_onearg(t->parent->jv, "apch1$", t->jv);
+/* but head and body link to document */
+	linked_in = false;
+	if (action == TAGACT_HEAD || action == TAGACT_BODY) {
+		run_function_onearg(cf->docobj, "eb$apch1", t->jv);
+		linked_in = true;
+	} else {
+		if (t->parent && t->parent->jv) {
+			run_function_onearg(t->parent->jv, "eb$apch1", t->jv);
+			linked_in = true;
+// special code for frame.contentDocument.
+			if (t->parent->action == TAGACT_FRAME) {
+				set_property_object(t->parent->jv,
+						    "contentDocument", t->jv);
+				set_property_object(t->parent->jv,
+						    "contentWindow", t->jv);
+			}
+		}
 
-	if (!t->parent) {
-		if (innerParent)
-			run_function_onearg(innerParent, "apch1$", t->jv);
-/* head and body link to document */
-		else if (action == TAGACT_HEAD || action == TAGACT_BODY)
-			run_function_onearg(cf->docobj, "apch1$", t->jv);
+		if (!t->parent && innerParent) {
+			run_function_onearg(innerParent, "eb$apch1", t->jv);
+			linked_in = true;
+		}
 	}
 
-/* TextNode linked to document/gc to protect if from garbage collection,
- * but now it is linked to its parent, and even if it isn't,
- * we don't need it hanging around anyways. */
-	if (action == TAGACT_TEXT)
+	if (linked_in && fakePropLast[0]) {
+// Node linked to document/gc to protect if from garbage collection,
+// but now it is linked to its parent.
 		delete_property(cf->docobj, fakePropLast);
+	}
 
 /* set innerHTML from the source html, if this tag supports it */
 	if (ti->bits & TAG_INNERHTML)
 		establish_inner(t->jv, t->innerHTML, 0, false);
+
+// If the tag has foo=bar as an attribute, pass this forward to javascript.
+	pushAttributes(t);
 }				/* jsNode */
 
+static void pushAttributes(const struct htmlTag *t)
+{
+	int i;
+	const char **a = t->attributes;
+	const char **v = t->atvals;
+	if (!a)
+		return;
+	for (i = 0; a[i]; ++i) {
+// There are some exceptions, some attributes that we handle individually.
+		const char *const exclist[] = {
+			"onclick", "onchange", "onsubmit", "onreset", "onload",
+			"onunload",
+			"name", "id",
+			"checked", "value", "type", "style",
+			"href", "src", "action",
+			0
+		};
+		const char *u;
+		if (stringInListCI(exclist, a[i]) >= 0)
+			continue;
+// I surely haven't thought of everything, so check generally.
+// Maybe they wrote <a firstChild=foo>
+// See if the name is specifically in the prototype.
+		if (has_property(t->jv, a[i]) && !typeof_property(t->jv, a[i])) {
+			debugPrint(3, "html attribute overload %s.%s",
+				   t->info->name, a[i]);
+			continue;
+		}
+// Should we drop attribute name to lower case? I don't, for now.
+		u = v[i];
+		if (!u)
+			u = emptyString;
+// There may be some, like multiple or readonly, that should be set to true,
+// not the empty string. I haven't dealt with those yet.
+// Should I call setAttribute, with its side effects, or is just setting
+// the property good enough??
+		set_property_string(t->jv, a[i], u);
+	}
+}				/* pushAttributes */
+
 /* decorate the tree of nodes with js objects */
 void decorate(int start)
 {
@@ -1375,6 +1550,7 @@ static void pushTag(struct htmlTag *t)
 {
 	int a = cw->allocTags;
 	if (cw->numTags == a) {
+		debugPrint(4, "%d tags, %d dead", a, cw->deadTags);
 /* make more room */
 		a = a / 2 * 3;
 		cw->tags =
@@ -1385,6 +1561,58 @@ static void pushTag(struct htmlTag *t)
 	tagCountCheck();
 }				/* pushTag */
 
+static void freeTag(struct htmlTag *t);
+
+// garbage collect the dead tags.
+// You must rerender after this runs, so that the buffer has no dead tags,
+// and the remaining tags have their new numbers embedded in the buffer.
+void tag_gc(void)
+{
+	int cx;			/* edbrowse context */
+	struct ebWindow *w, *save_cw;
+	struct htmlTag *t;
+	int i, j;
+
+	for (cx = 1; cx <= maxSession; ++cx) {
+		for (w = sessionList[cx].lw; w; w = w->prev) {
+			if (!w->tags)
+				continue;
+// Don't bother unless a third of the tags are dead.
+			if (w->deadTags * 3 < w->numTags)
+				continue;
+
+// sync any changed fields before we muck with the tags.
+			save_cw = cw;
+			cw = w;
+			cf = &(cw->f0);
+			jSyncup(true);
+			cw = save_cw;
+			cf = &(cw->f0);
+
+// ok let's crunch.
+			for (i = j = 0; i < w->numTags; ++i) {
+				t = w->tags[i];
+				if (t->dead) {
+					freeTag(t);
+				} else {
+					t->seqno = j;
+					w->tags[j++] = t;
+				}
+			}
+			debugPrint(4, "tag_gc from %d to %d", w->numTags, j);
+			w->numTags = j;
+			w->deadTags = 0;
+
+// We must rerender when we return to this window,
+// or at the input loop if this is the current window.
+// Tags have been renumbered, need to rebuild the text buffer accordingly.
+			w->mustrender = true;
+			if (w != cw)
+				w->nextrender = 0;
+		}
+	}
+}
+
 /* first three have to be in this order */
 const struct tagInfo availableTags[] = {
 	{"html", "html", TAGACT_ZERO},
@@ -1401,6 +1629,8 @@ const struct tagInfo availableTags[] = {
 	{"sup", "a superscript", TAGACT_SUP, 0, 0},
 	{"ovb", "an overbar", TAGACT_OVB, 0, 0},
 	{"font", "a font", TAGACT_NOP, 0, 0},
+	{"cite", "a citation", TAGACT_NOP, 0, 0},
+	{"tt", "teletype", TAGACT_NOP, 0, 0},
 	{"center", "centered text", TAGACT_P, 2, 5},
 	{"caption", "a caption", TAGACT_NOP, 5, 0},
 	{"head", "the html header information", TAGACT_HEAD, 0, 5},
@@ -1409,14 +1639,18 @@ const struct tagInfo availableTags[] = {
 	{"bgsound", "background music", TAGACT_MUSIC, 0, 4},
 	{"audio", "audio passage", TAGACT_MUSIC, 0, 4},
 	{"meta", "a meta tag", TAGACT_META, 0, 4},
+	{"style", "a style tag", TAGACT_STYLE, 0, 2},
 	{"link", "a link tag", TAGACT_LINK, 0, 4},
 	{"img", "an image", TAGACT_IMAGE, 0, 4},
 	{"image", "an image", TAGACT_IMAGE, 0, 4},
 	{"br", "a line break", TAGACT_BR, 1, 4},
 	{"p", "a paragraph", TAGACT_P, 2, 5},
+	{"header", "a header", TAGACT_HEADER, 2, 5},
+	{"footer", "a footer", TAGACT_FOOTER, 2, 5},
 	{"div", "a divided section", TAGACT_DIV, 5, 1},
 	{"map", "a map of images", TAGACT_NOP, 5, 0},
 	{"blockquote", "a quoted paragraph", TAGACT_NOP, 10, 1},
+	{"document", "a document", TAGACT_NOP, 5, 1},
 	{"h1", "a level 1 header", TAGACT_NOP, 10, 1},
 	{"h2", "a level 2 header", TAGACT_NOP, 10, 1},
 	{"h3", "a level 3 header", TAGACT_NOP, 10, 1},
@@ -1433,7 +1667,7 @@ const struct tagInfo availableTags[] = {
 	{"dl", "a definition list", TAGACT_DL, 10, 1},
 	{"hr", "a horizontal line", TAGACT_HR, 5, 4},
 	{"form", "a form", TAGACT_FORM, 10, 1},
-	{"button", "a button", TAGACT_INPUT, 0, 4},
+	{"button", "a button", TAGACT_INPUT, 0, 0},
 	{"frame", "a frame", TAGACT_FRAME, 2, 0},
 	{"iframe", "a frame", TAGACT_FRAME, 2, 1},
 	{"map", "an image map", TAGACT_MAP, 2, 4},
@@ -1452,7 +1686,7 @@ const struct tagInfo availableTags[] = {
 	{"address", "an address block", TAGACT_NOP, 1, 0},
 	{"style", "a style block", TAGACT_NOP, 0, 2},
 	{"script", "a script", TAGACT_SCRIPT, 0, 0},
-	{"noscript", "no script section", TAGACT_NOP, 0, 2},
+	{"noscript", "no script section", TAGACT_NOSCRIPT, 0, 2},
 	{"noframes", "no frames section", TAGACT_NOP, 0, 2},
 	{"embed", "embedded html", TAGACT_MUSIC, 0, 4},
 	{"noembed", "no embed section", TAGACT_NOP, 0, 2},
@@ -1472,6 +1706,40 @@ const struct tagInfo availableTags[] = {
 	{"", NULL, 0}
 };
 
+static void freeTag(struct htmlTag *t)
+{
+	char **a;
+	nzFree(t->textval);
+	nzFree(t->name);
+	nzFree(t->id);
+	nzFree(t->value);
+	cnzFree(t->rvalue);
+	nzFree(t->href);
+	nzFree(t->class);
+	nzFree(t->js_file);
+	nzFree(t->innerHTML);
+
+	a = (char **)t->attributes;
+	if (a) {
+		while (*a) {
+			nzFree(*a);
+			++a;
+		}
+		free(t->attributes);
+	}
+
+	a = (char **)t->atvals;
+	if (a) {
+		while (*a) {
+			nzFree(*a);
+			++a;
+		}
+		free(t->atvals);
+	}
+
+	free(t);
+}
+
 void freeTags(struct ebWindow *w)
 {
 	int i, n;
@@ -1496,42 +1764,14 @@ void freeTags(struct ebWindow *w)
 
 	e = w->tags;
 	for (i = 0; i < w->numTags; ++i, ++e) {
-		char **a;
 		t = *e;
-		nzFree(t->textval);
-		nzFree(t->name);
-		nzFree(t->id);
-		nzFree(t->value);
-		cnzFree(t->rvalue);
-		nzFree(t->href);
-		nzFree(t->classname);
-		nzFree(t->js_file);
-		nzFree(t->innerHTML);
-
-		a = (char **)t->attributes;
-		if (a) {
-			while (*a) {
-				nzFree(*a);
-				++a;
-			}
-			free(t->attributes);
-		}
-
-		a = (char **)t->atvals;
-		if (a) {
-			while (*a) {
-				nzFree(*a);
-				++a;
-			}
-			free(t->atvals);
-		}
-
-		free(t);
+		disconnectTagObject(t);
+		freeTag(t);
 	}
 
 	free(w->tags);
 	w->tags = 0;
-	w->numTags = w->allocTags = 0;
+	w->numTags = w->allocTags = w->deadTags = 0;
 }				/* freeTags */
 
 struct htmlTag *newTag(const char *name)
@@ -1545,7 +1785,7 @@ struct htmlTag *newTag(const char *name)
 			break;
 
 	if (!ti->name[0]) {
-		debugPrint(3, "warning, created node %s reverts to generic",
+		debugPrint(4, "warning, created node %s reverts to generic",
 			   name);
 		ti = availableTags + 2;
 	}
@@ -1567,6 +1807,7 @@ void initTagArray(void)
 {
 	cw->numTags = 0;
 	cw->allocTags = 512;
+	cw->deadTags = 0;
 	cw->tags =
 	    (struct htmlTag **)allocMem(cw->allocTags *
 					sizeof(struct htmlTag *));
@@ -1606,15 +1847,20 @@ static void intoTree(struct htmlTag *parent)
 	while (tree_pos < cw->numTags) {
 		t = tagList[tree_pos++];
 		if (t->slash) {
-			if (parent)
+			if (parent) {
 				parent->balance = t, t->balance = parent;
+				t->dead = parent->dead;
+				if (t->dead)
+					++cw->deadTags;
+			}
 			debugPrint(4, "}");
 			return;
 		}
 
 		if (treeDisable) {
 			debugPrint(4, "node skip %s", t->info->name);
-			t->step = 100;
+			t->dead = true;
+			++cw->deadTags;
 			intoTree(t);
 			continue;
 		}
@@ -1627,7 +1873,8 @@ static void intoTree(struct htmlTag *parent)
 			action = t->action;
 			if (action == TAGACT_HEAD) {
 				debugPrint(4, "node skip %s", t->info->name);
-				t->step = 100;
+				t->dead = true;
+				++cw->deadTags;
 				treeDisable = true;
 				intoTree(t);
 				treeDisable = false;
@@ -1635,7 +1882,8 @@ static void intoTree(struct htmlTag *parent)
 			}
 			if (action == TAGACT_BODY) {
 				debugPrint(4, "node pass %s", t->info->name);
-				t->step = 100;
+				t->dead = true;
+				++cw->deadTags;
 				intoTree(t);
 				continue;
 			}
@@ -1696,6 +1944,8 @@ checkattributes:
 			t->checked = t->rchecked = true;
 		if (stringInListCI(t->attributes, "readonly") >= 0)
 			t->rdonly = true;
+		if (stringInListCI(t->attributes, "disabled") >= 0)
+			t->disabled = true;
 		if (stringInListCI(t->attributes, "multiple") >= 0)
 			t->multiple = true;
 		if ((j = stringInListCI(t->attributes, "name")) >= 0) {
@@ -1715,7 +1965,7 @@ checkattributes:
 			v = t->atvals[j];
 			if (v && !*v)
 				v = 0;
-			t->classname = cloneString(v);
+			t->class = cloneString(v);
 		}
 		if ((j = stringInListCI(t->attributes, "value")) >= 0) {
 			v = t->atvals[j];
@@ -1724,6 +1974,12 @@ checkattributes:
 			t->value = cloneString(v);
 			t->rvalue = cloneString(v);
 		}
+// Resolve href against the base, but wait a minute, what if it's <p href=blah>
+// and we're not suppose to resolve it? I don't ask about the parent node.
+// Well, in general, I don't carry the href attribute into the js node.
+// I only do it when it is relevant, such as <a> or <area>.
+// See the exceptions in pushAttributes() in this file.
+// I know, it's confusing.
 		if ((j = stringInListCI(t->attributes, "href")) >= 0) {
 			v = t->atvals[j];
 			if (v && !*v)
@@ -1778,18 +2034,80 @@ checkattributes:
 	}
 }				/* intoTree */
 
-/* Parse some html, as generated by innerHTML or document.write.
- * This assumes in_js_cw has been set up properly,
- * with the correct winobj and docobj etc. */
+void underKill(struct htmlTag *t)
+{
+	struct htmlTag *u, *v;
+	for (u = t->firstchild; u; u = v) {
+		v = u->sibling;
+		u->sibling = u->parent = 0;
+		u->deleted = true;
+		if (!u->jv)
+			killTag(u);
+	}
+	t->firstchild = NULL;
+}
+
+void killTag(struct htmlTag *t)
+{
+	struct htmlTag *c, *parent;
+	debugPrint(4, "kill tag %s %d", t->info->name, t->seqno);
+	t->dead = true;
+	++cw->deadTags;
+	if (t->balance) {
+		t->balance->dead = true;
+		++cw->deadTags;
+	}
+	t->deleted = true;
+	t->jv = NULL;
+
+// unlink it from the tree above.
+	parent = t->parent;
+	if (parent) {
+		t->parent = NULL;
+		if (parent->firstchild == t)
+			parent->firstchild = t->sibling;
+		else {
+			c = parent->firstchild;
+			if (c) {
+				for (; c->sibling; c = c->sibling) {
+					if (c->sibling != t)
+						continue;
+					c->sibling = t->sibling;
+					break;
+				}
+			}
+		}
+	}
+
+	underKill(t);
+}
+
+/* Parse some html, as generated by innerHTML or document.write. */
 void html_from_setter(jsobjtype inner, const char *h)
 {
-	initTagArray();
+	struct htmlTag *t = NULL;
+	int l = 0;
+	debugPrint(4, "Generated {%s}", h);
+	t = tagFromJavaVar(inner);
+	if (!t) {
+		debugPrint(1,
+			   "innerHTML finds no tag for %p, cannot parse",
+			   inner);
+		return;
+	}
+	debugPrint(4, "parse under %s %d", t->info->name, t->seqno);
+	l = cw->numTags;
+
+/* Cut all the children away from t */
+	underKill(t);
+
 	html2nodes(h, false);
 	htmlGenerated = true;
-	htmlNodesIntoTree(0, NULL);
+	htmlNodesIntoTree(l, t);
 	prerender(0);
 	innerParent = inner;
 	decorate(0);
+	innerParent = 0;
 }				/* html_from_setter */
 
 static void processStyles(jsobjtype so, const char *stylestring)
@@ -1815,8 +2133,17 @@ static void processStyles(jsobjtype so, const char *stylestring)
 			trimWhite(s);
 			trimWhite(sv);
 // the property name has to be nonempty
-			if (*s)
+			if (*s) {
+// foo-bar has to become fooBar
+				char *t, *w;
+				for (t = w = s; *t; ++t)
+					if (*t == '-' && isalpha(t[1]))
+						t[1] = toupper(t[1]);
+					else
+						*w++ = *t;
+				*w = 0;
 				set_property_string(so, s, sv);
+			}
 		}
 	}
 	nzFree(workstring);
