@@ -27,6 +27,7 @@
 
 /* seems like everybody needs these header files */
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <ctype.h>
 #include <string.h>
 #include <signal.h>
@@ -42,6 +43,7 @@
 #include <direct.h> // for _mkdir, ...
 #include <conio.h>  // for _kbhit, getch, getche
 #include <stdint.h> // for UINT32_MAX
+#include "vsprtf.h" // for WIN32 asprintf, vasprintf, ...
 #else
 #include <unistd.h>
 #endif
@@ -84,17 +86,27 @@ typedef uchar bool;
 extern "C" {
 #endif
 
-/*********************************************************************
-Include the header file that connects edbrowse to the js process.
-This is a series of enums, and the interprocess message structure,
-and most importantly, the type jsobjtype,
-which is an opaque number that corresponds to an object in the javascript world.
-Edbrowse uses this number to connect an html tag, such as <input>,
-with its corresponding js object.
-These object numbers are passed back and forth to connect edbrowse and js.
-*********************************************************************/
+// Opaque indicator of an object that can be shared
+// between edbrowse and the js engine.
+typedef void *jsobjtype;
 
-#include "ebjs.h"
+extern jsobjtype jcx; // the javascript context
+extern jsobjtype winobj;	// window object
+extern jsobjtype docobj;	// document object
+extern const char *jsSourceFile; // sourcefile providing the javascript
+extern int jsLineno; // line number
+
+enum ej_proptype {
+	EJ_PROP_NONE,
+	EJ_PROP_STRING,
+	EJ_PROP_BOOL,
+	EJ_PROP_INT,
+	EJ_PROP_FLOAT,
+	EJ_PROP_OBJECT,
+	EJ_PROP_ARRAY,
+	EJ_PROP_FUNCTION,
+	EJ_PROP_INSTANCE,
+};
 
 /* ctype macros, when you're passing a byte,
  * and you don't want to worry about whether it's char or uchar.
@@ -126,26 +138,17 @@ typedef uchar *pst;		/* perl string */
 #define TableCellChar '\3'
 
 /* How long can an absolute path be? */
-#define ABSPATH 264 // max length of an absolute pathname, in windows or unix
-/* How long can a regular expression be? */
-#define MAXRE 400
-/* How long can an entered line be? */
-#define MAXTTYLINE 256
-/* The longest string, in certain contexts. */
-#define MAXSTRLEN 1024
-/* How about user login and password? */
-#define MAXUSERPASS 40
-/* Number of pop3 mail accounts */
-#define MAXACCOUNT 100
-/* Number of mime types */
-#define MAXMIME 40
-/* Number of proxy entries */
-#define MAXPROXY 200
-/* number of db tables */
-#define MAXDBT 100
-#define MAXTCOLS 40
-/* How many sessions open concurrently */
-#define MAXSESSION 1000
+#define ABSPATH 1024 // max length of an absolute pathname
+#define MAXRE 512 // max length of a regular expression
+#define MAXTTYLINE 256 // max length of an entered line
+#define MAXUSERPASS 40 // user name or password
+#define MAXACCOUNT 100 // number of email accounts
+#define MAXAGENT 50 // number of user agents
+#define MAXMIME 40 // number of mime types
+#define MAXPROXY 200 // number of proxy entries
+#define MAXDBT 100 // number of configured database tables
+#define MAXTCOLS 40 // columns in a configured table
+#define MAXSESSION 1000 // concurrent edbrowse sessions
 /* Allocation increment for a growing string, that we don't expect
  * to get too large.  This must be a power of 2. */
 #define ALLOC_GR        0x100
@@ -177,7 +180,7 @@ extern char emptyString[];	/* use this whenever you would use "" */
  * door unless you are one of three approved browsers.
  * Tell them you're Explorer, and walk right in.
  * Anyways, this array holds up to 10 user agent strings. */
-extern char *userAgents[10], *currentAgent;
+extern char *userAgents[], *currentAgent;
 extern char *newlocation;
 extern int newloc_d; /* delay */
 extern bool newloc_r; /* location replaces this page */
@@ -239,20 +242,25 @@ struct DBTABLE {
 extern CURL *global_http_handle;
 extern CURLSH *global_share_handle;
 extern int debugLevel;		/* 0 to 9 */
+extern bool debugClone, debugEvent, debugThrow;
+extern bool demin; // deminimize javascript
+extern bool gotimers; // run javascript timers
 extern FILE *debugFile;
+extern char *debugFileName;
 extern char *sslCerts;		/* ssl certificates to validate the secure server */
 extern int verifyCertificates;	/* is a certificate required for the ssl connection? */
 extern int displayLength;	/* when printing a line */
-extern int jsPool;		/* size of js pool in megabytes */
 extern int webTimeout, mailTimeout;
 extern uchar browseLocal;
 extern bool sqlPresent;		/* Was edbrowse compiled with SQL built in? */
+extern bool curlActive; // is curl running?
 extern bool ismc;		/* Is the program running as a mail client? */
 extern bool isimap;		/* Is the program running as an imap client? */
 extern bool down_bg;		/* download in background */
 extern char whichproc; // which edbrowse-xx process
 extern char showProgress; // feedback as a file is downloaded
-extern int eb_lang;		/* edbrowse language, determined by $LANG */
+extern char eb_language[];		/* edbrowse language, determined by $LANG */
+extern int eb_lang; // encoded version of the above, for languages that we recognize
 extern bool cons_utf8;		/* does the console expect utf8? */
 extern bool iuConvert;		/* perform iso utf8 conversions automatically */
 extern char type8859;		/* 1 through 15 */
@@ -263,13 +271,12 @@ extern bool isInteractive;
 extern volatile bool intFlag;	/* set this when interrupt signal is caught */
 extern bool binaryDetect;
 extern bool inputReadLine;
+extern bool curlAuthNegotiate;  /* try curl negotiate (SPNEGO) auth */
 extern bool listNA;		/* list nonascii chars */
 extern bool inInput;		/* reading line from standard in */
 extern int fileSize;		/* when reading/writing files */
-extern int mssock;		/* mail server socket */
 extern long ht_code;		/* http code, like 404 file not found */
 extern char errorMsg[];		/* generated error message */
-extern char serverLine[];	/* lines to and from the mail server */
 extern int localAccount;	/* this is the smtp server for outgoing mail */
 extern char *mailDir;		/* move to this directory when fetching mail */
 extern char *mailUnread;	/* place to hold fetched but unread mail */
@@ -321,26 +328,6 @@ struct listHead {
 (e) != (void*)&(l); \
 (e) = ((struct listHead *)e)->prev)
 
-/*********************************************************************
-a queue of input field values that have been changed by javascript.
-these are applied to the edbrowse buffer after js has run.
-Other changes can also accumulate in this queue, such as innerHTML.
-The major number indicates the change to be made.
-v = value in form, i is innerHTML or innerText (via minor),
-x is unspecified.
-*********************************************************************/
-
-struct inputChange {
-	struct inputChange *next, *prev;
-	struct htmlTag *t;
-	int tagno;
-	char major, minor;
-	char filler1, filler2;
-	struct ebFrame *f0;
-	char value[4];
-};
-extern struct listHead inputChangesPending;
-
 /* A pointer to the text of a line, and other line attributes */
 struct lineMap {
 	pst text;
@@ -362,10 +349,13 @@ struct ebFrame {
 	char *fileName;		/* name of file or url */
 	char *firstURL;		/* before http redirection */
 	char *hbase; /* base for href references */
-	bool baseset; /* <base> tag has been seen */
-	bool f_encoded:1; /* filename is url encoded */
+	bool baseset; // <base> tag has been seen
+	bool f_encoded; // filename is url encoded
+	bool fromcurl; // data comes from the internet
 	char *dw;		/* document.write string */
 	int dw_l;		/* length of the above */
+// document.writes go under the body.
+	struct htmlTag *bodytag;
 /* The javascript context and window corresponding to this url or frame.
  * If this is null then javascript is not operational for this frame.
  * We could still be browsing however, without javascript. */
@@ -387,6 +377,7 @@ struct ebWindow {
 /* remember dot and dol for the raw text, when in browse mode */
 	int r_dot, r_dol;
 	struct ebFrame f0; /* first frame */
+	struct ebFrame *jdb_frame; // if in jdb mode
 /* is the referrer the original web page or the individual frame? */
 	char *referrer;
 	char *baseDirName;	/* when scanning a directory */
@@ -405,7 +396,7 @@ struct ebWindow {
 /* Next is an array of html tags, generated by the browse command,
  * and used thereafter for hyperlinks, fill-out forms, etc. */
 	struct htmlTag **tags;
-	int numTags, allocTags;
+	int numTags, allocTags, deadTags;
 	bool mustrender:1;
 	bool sank:1; /* jSyncup has been run */
 	bool lhs_yes:1;
@@ -455,6 +446,7 @@ struct ebSession {
 };
 extern struct ebSession sessionList[];
 extern struct ebSession *cs;	/* current session */
+extern int maxSession;
 
 /* The information on an html tag */
 #define MAXTAGNAME 12
@@ -502,9 +494,11 @@ struct htmlTag {
 	bool slash:1;		/* as in </A> */
 	bool textin:1; /* <a> some text </a> */
 	bool deleted:1; /* deleted from the current buffer */
+	bool dead:1; // removed by garbage collection
 	bool contracted:1; /* frame is contracted */
 	bool multiple:1;
 	bool rdonly:1;
+	bool disabled:1;
 	bool clickable:1;	/* but not an input field */
 	bool secure:1;
 	bool scriptgen:1; // script generated, not from source
@@ -522,15 +516,15 @@ struct htmlTag {
 	bool onload:1;
 	bool onunload:1;
 	bool doorway:1; /* doorway to javascript */
-	bool visited: 1;
+	bool visited:1;
+	bool masked:1;
 	char subsup;		/* span turned into sup or sub */
 	uchar itype;		/* input type = */
+	uchar itype_minor;
 	int ninp;		/* number of nonhidden inputs */
 	char *name, *id, *value, *href;
 	const char *rvalue; /* for reset */
-/* class=foo becomes className = "foo" when you carry from html to javascript,
- * don't ask me why. */
-	char *classname;
+	char *class;
 	char *innerHTML; /* the html string under this tag */
 	int inner;		/* for inner html */
 };
@@ -545,9 +539,11 @@ enum {
 	TAGACT_MUSIC, TAGACT_IMAGE, TAGACT_BR, TAGACT_IBR, TAGACT_P,
 	TAGACT_BASE, TAGACT_META, TAGACT_LINK, TAGACT_PRE, TAGACT_TBODY,
 	TAGACT_DT, TAGACT_DD, TAGACT_LI, TAGACT_TABLE, TAGACT_TR, TAGACT_TD,
-	TAGACT_DIV, TAGACT_SPAN, TAGACT_HR, TAGACT_OBJECT,
-	TAGACT_FORM, TAGACT_FRAME,
-	TAGACT_MAP, TAGACT_AREA, TAGACT_SCRIPT, TAGACT_EMBED, TAGACT_OBJ,
+	TAGACT_DIV, TAGACT_SPAN, TAGACT_HR, TAGACT_OBJECT, TAGACT_FOOTER,
+	TAGACT_HEADER, // <header> tag, not the same as <head> tag
+	TAGACT_FORM, TAGACT_FRAME, TAGACT_STYLE,
+	TAGACT_MAP, TAGACT_AREA, TAGACT_SCRIPT, TAGACT_NOSCRIPT, TAGACT_EMBED,
+	TAGACT_OBJ,
 };
 
 /* htmlTag.itype */
@@ -555,10 +551,19 @@ enum {
 /* Corresponds to inp_types in decorate.c */
 enum {
 	INP_RESET, INP_BUTTON, INP_IMAGE, INP_SUBMIT,
-	INP_HIDDEN,
-	INP_TEXT, INP_PW, INP_NUMBER, INP_FILE,
+	INP_HIDDEN, INP_TEXT, INP_FILE,
 	INP_SELECT, INP_TA, INP_RADIO, INP_CHECKBOX,
 };
+extern const char *const inp_types[];
+
+/* htmlTag.itype_minor */
+/* The order corresponds to inp_others in decorate.c */
+enum {
+	INP_NO_MINOR, INP_DATE, INP_DATETIME, INP_DATETIME_LOCAL,
+	INP_MONTH, INP_WEEK, INP_TIME, INP_EMAIL, INP_RANGE,
+	INP_SEARCH, INP_TEL, INP_URL, INP_NUMBER, INP_PW,
+};
+extern const char *const inp_others[];
 
 /* For traversing a tree of html nodes, this is the callback function */
 typedef void (*nodeFunction) (struct htmlTag * node, bool opentag);
