@@ -2,7 +2,9 @@
 
 /* sourcefile=auth.c */
 bool getUserPass(const char *url, char *creds, bool find_proxy) ;
-bool addWebAuthorization(const char *url, const char *credentials, bool proxy) ;
+bool getUserPassRealm(const char *url, char *creds, const char *realm);
+bool addWebAuthorization(const char *url, const char *credentials, bool proxy,
+			 const char *realm);
 
 /* sourcefile=buffers.c */
 void removeHiddenNumbers(pst p, uchar terminate);
@@ -33,6 +35,7 @@ bool domainSecurityCheck(const char *server, const char *domain) ;
 bool receiveCookie(const char *url, const char *str) ;
 void cookiesFromJar(void) ;
 void sendCookies(char **s, int *l, const char *url, bool issecure) ;
+void mergeCookies(void);
 
 /* sourcefile=cache.c */
 void setupEdbrowseCache(void);
@@ -60,14 +63,16 @@ bool ebConnect(void) ;
 int goSelect(int *startLine, char **rbuf) ;
 
 /* sourcefile=ebjs.c */
+void javaSetsTagVar(jsobjtype v, const char *newtext);
+void javaSetsInner(jsobjtype v, const char *newtext);
 void dwStart(void);
 void createJavaContext(void) ;
 void freeJavaContext(struct ebFrame *f) ;
-void js_shutdown(void) ;
-void js_disconnect(void);
 char *jsRunScriptResult(jsobjtype obj, const char *str, const char *filename, int lineno) ;
 void jsRunScript(jsobjtype obj, const char *str, const char *filename, int lineno) ;
-enum ej_proptype has_property(jsobjtype obj, const char *name) ;
+enum ej_proptype typeof_property(jsobjtype obj, const char *name) ;
+bool has_property(jsobjtype obj, const char *name) ;
+#define handlerPresent(obj, name) (typeof_property(obj, name) == EJ_PROP_FUNCTION)
 void delete_property(jsobjtype obj, const char *name) ;
 char *get_property_string(jsobjtype obj, const char *name) ;
 int get_property_number(jsobjtype obj, const char *name) ;
@@ -87,14 +92,12 @@ jsobjtype instantiate_array_element(jsobjtype array, int idx, const char *classn
 jsobjtype instantiate(jsobjtype parent, const char *name, const char *classname) ;
 int set_property_function(jsobjtype parent, const char *name, const char *body) ;
 int get_arraylength(jsobjtype a);
-void update_var_in_js(int varid);
 char *get_property_option(jsobjtype obj) ;
 void setupJavaDom(void) ;
 char *get_property_url(jsobjtype owner, bool action) ;
 void rebuildSelectors(void);
-jsobjtype run_function_object(jsobjtype obj, const char *name);
 bool run_function_bool(jsobjtype obj, const char *name);
-void run_function_objargs(jsobjtype obj, const char *name, int nargs, ...);
+bool run_event_bool(jsobjtype obj, const char *pname, const char *eevname);
 void run_function_onearg(jsobjtype obj, const char *name, jsobjtype o);
 void set_basehref(const char *b);
 
@@ -116,13 +119,14 @@ char *htmlReformat(char *buf) ;
 void extractEmailAddresses(char *line) ;
 void cutDuplicateEmails(char *tolist, char *cclist, const char *reply) ;
 int byteOrderMark(const uchar *buf, int buflen);
-bool looksBinary(const char *buf, int buflen);
-void looks_8859_utf8(const char *buf, int buflen, bool * iso_p, bool * utf8_p);
-void iso2utf(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p);
-void utf2iso(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p);
+bool looksBinary(const unsigned char *buf, int buflen);
+void looks_8859_utf8(const uchar *buf, int buflen, bool * iso_p, bool * utf8_p);
+void iso2utf(const uchar *inbuf, int inbuflen, uchar **outbuf_p, int *outbuflen_p);
+void utf2iso(const uchar *inbuf, int inbuflen, uchar **outbuf_p, int *outbuflen_p);
 void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p, bool inutf8, bool out32, bool outbig);
 char *uni2utf8(unsigned int unichar);
 void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p, int bom);
+char *force_utf8( char *buf, int buflen);
 char *base64Encode(const char *inbuf, int inlen, bool lines);
 void iuReformat(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p) ;
 bool parseDataURI(const char *uri, char **mediatype, char **data, int *data_l);
@@ -141,17 +145,16 @@ void infShow(int tagno, const char *search) ;
 bool infReplace(int tagno, const char *newtext, bool notify) ;
 bool infPush(int tagno, char **post_string) ;
 struct htmlTag *tagFromJavaVar(jsobjtype v);
-struct htmlTag *tagFromJavaVar2(jsobjtype v, const char *tagname);
 void javaSubmitsForm(jsobjtype v, bool reset) ;
-bool handlerGoBrowse(const struct htmlTag *t, const char *name) ;
+bool bubble_event(const struct htmlTag *t, const char *name) ;
 void runningError(int msg, ...) ;
 void rerender(bool notify);
 void delTags(int startRange, int endRange);
 extern void runOnload(void);
+void javaSetsTimeout(int n, const char *jsrc, jsobjtype to, bool isInterval);
 bool timerWait(int *delay_sec, int *delay_ms);
 void delTimers(struct ebFrame *f);
-void delInputChanges(struct ebFrame *f);
-void runTimers(void);
+void runTimer(void);
 void javaOpensWindow(const char *href, const char *name) ;
 void javaSetsLinkage(bool after, char type, jsobjtype p, const char *rest);
 
@@ -173,7 +176,11 @@ void decorate(int start);
 void freeTags(struct ebWindow *w) ;
 struct htmlTag *newTag(const char *tagname) ;
 void initTagArray(void);
+void tag_gc(void);
+void tag_gc(void);
 void htmlNodesIntoTree(int start, struct htmlTag *attach);
+void underKill(struct htmlTag *t);
+void killTag(struct htmlTag *t);
 void html_from_setter( jsobjtype innerParent, const char *h);
 
 /* sourcefile=http.c */
@@ -188,6 +195,7 @@ bool shortRefreshDelay(void);
 bool httpConnect(const char *url, bool down_ok, bool webpage, bool f_encoded, char **headers_p, char **body_p, int *bodlen_p);
 void ebcurl_setError(CURLcode curlret, const char *url) ;
 void setHTTPLanguage(const char *lang) ;
+int prompt_and_read(int prompt, char *buffer, int buffer_length, int error_message, bool hide_echo);
 int ebcurl_debug_handler(CURL * handle, curl_infotype info_desc, char *data, size_t size, void *unused) ;
 int bg_jobs(bool iponly);
 void addNovsHost(char *host) ;
@@ -195,8 +203,10 @@ void deleteNovsHosts(void);
 CURLcode setCurlURL(CURL * h, const char *url) ;
 const char *findProxyForURL(const char *url) ;
 bool frameExpand(bool expand, int ln1, int ln2);
+int frameExpandLine(int ln, jsobjtype fo);
 struct htmlTag *line2frame(int ln);
 bool reexpandFrame(void);
+bool frameSecurityFile(const char *thisfile);
 
 /* sourcefile=main.c */
 const char *mailRedirect(const char *to, const char *from, const char *reply, const char *subj) ;
@@ -246,7 +256,6 @@ void i_caseShift(unsigned char *s, char action) ;
 #endif
 void eeCheck(void) ;
 void eb_puts(const char *s);
-void eb_vprintf (const char *fmt, va_list args) ;
 
 /* sourcefile=stringfile.c */
 void *allocMem(size_t n) ;
@@ -320,6 +329,7 @@ char *conciseTime(time_t t);
 bool lsattrChars(const char *buf, char *dest);
 char *lsattr(const char *path, const char *flags);
 void ttySaveSettings(void) ;
+void ttySetEcho(bool enable_echo);
 #ifndef _INC_CONIO
 int getche(void) ;
 int getch(void) ;
@@ -328,8 +338,8 @@ char getLetter(const char *s) ;
 char *getFileName(int msg, const char *defname, bool isnew, bool ws);
 int shellProtectLength(const char *s);
 void shellProtect(char *t, const char *s);
-const char *nextScanFile(const char *base) ;
-bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p) ;
+const char *nextScanFile(const char *base);
+bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p, int othersort, bool reverse) ;
 bool envFile(const char *line, const char **expanded);
 bool envFileDown(const char *line, const char **expanded) ;
 FILE *efopen(const char *name, const char *mode) ;
@@ -368,13 +378,22 @@ char *encodePostData(const char *s) ;
 char *decodePostData(const char *data, const char *name, int seqno) ;
 void decodeMailURL(const char *url, char **addr_p, char **subj_p, char **body_p) ;
 
-/* sourcefile=jseng-moz.cpp */
-int js_main(int argc, char **argv);
+/* sourcefile=jseng-duk.c */
+void connectTagObject(struct htmlTag *t, jsobjtype p);
+void disconnectTagObject(struct htmlTag *t);
+int js_main(void);
 // the native versions of the api functions in ebjs.c
-enum ej_proptype has_property_nat(jsobjtype obj, const char *name) ;
+void createJavaContext_nat(void);
+void freeJavaContext_nat(void);
+enum ej_proptype typeof_property_nat(jsobjtype obj, const char *name) ;
+bool has_property_nat(jsobjtype obj, const char *name) ;
 void delete_property_nat(jsobjtype obj, const char *name) ;
 char *get_property_string_nat(jsobjtype obj, const char *name) ;
+int get_property_number_nat(jsobjtype parent, const char *name) ;
+double get_property_float_nat(jsobjtype parent, const char *name) ;
+bool get_property_bool_nat(jsobjtype parent, const char *name) ;
 jsobjtype get_property_object_nat(jsobjtype parent, const char *name) ;
+jsobjtype get_property_function_nat(jsobjtype parent, const char *name) ;
 jsobjtype get_array_element_object_nat(jsobjtype obj, int idx) ;
 int set_property_string_nat(jsobjtype obj, const char *name, const char *value) ;
 int set_property_number_nat(jsobjtype obj, const char *name, int n) ;
@@ -387,5 +406,7 @@ jsobjtype instantiate_array_element_nat(jsobjtype array, int idx, const char *cl
 jsobjtype instantiate_nat(jsobjtype parent, const char *name, const char *classname) ;
 int set_property_function_nat(jsobjtype parent, const char *name, const char *body) ;
 int get_arraylength_nat(jsobjtype a);
+bool run_function_bool_nat(jsobjtype obj, const char *name);
 void run_function_onearg_nat(jsobjtype obj, const char *name, jsobjtype o);
+char *run_script_nat(const char *s);
 
