@@ -55,6 +55,7 @@ static char *ht_cdfn;		/* http content disposition file name */
 static time_t ht_modtime;	/* http modification time */
 static char *ht_etag;		/* the etag in the header */
 static bool ht_cacheable;
+static char ht_auth_realm[60];	/* WWW-Authenticate realm header */
 
 /*********************************************************************
 This function is called for a new web page, by http refresh,
@@ -152,7 +153,7 @@ static void scan_http_headers(bool fromCallback)
 
 	if (!ht_cdfn && (v = find_http_header("content-disposition"))) {
 		char *s = strstrCI(v, "filename=");
-		if (s) {
+		if (s && !strncmp(v, "attachment", 10)) {
 			s += 9;
 			if (*s == '"') {
 				char *t;
@@ -203,6 +204,31 @@ static void scan_http_headers(bool fromCallback)
 			debugPrint(3, "mod date %s", v);
 		nzFree(v);
 	}
+	if (!ht_auth_realm[0] && (v = find_http_header("WWW-Authenticate"))) {
+		char *realm, *end;
+		if ((realm = strstrCI(v, "realm="))) {
+			realm += 6;
+			if (strchr("'\"", realm[0])) {
+				end = strchr(realm + 1, realm[0]);
+				realm++;
+			} else {
+				/* look for space if unquoted */
+				end = strchr(realm, ' ');
+			}
+			if (end) {
+				int sz = end - realm;
+				if (sz > sizeof(ht_auth_realm) - 1)
+					sz = sizeof(ht_auth_realm) - 1;
+				memcpy(ht_auth_realm, realm, sz);
+				ht_auth_realm[sz] = 0;
+			} else {
+				strncpy(ht_auth_realm, realm,
+					sizeof(ht_auth_realm) - 1);
+			}
+			debugPrint(3, "auth realm %s", ht_auth_realm);
+		}
+		nzFree(v);
+	}
 
 	if (fromCallback)
 		return;
@@ -212,7 +238,7 @@ static void scan_http_headers(bool fromCallback)
 		gotoLocation(v, -1, true);
 	}
 
-	if (v = find_http_header("refresh")) {
+	if ((v = find_http_header("refresh"))) {
 		int delay;
 		if (parseRefresh(v, &delay)) {
 			unpercentURL(v);
@@ -243,6 +269,7 @@ static CURLcode fetch_internet(CURL * h)
 	ht_etag = NULL;
 	ht_length = 0;
 	ht_modtime = 0;
+	ht_auth_realm[0] = 0;
 	curlret = curl_easy_perform(h);
 	if (is_http)
 		scan_http_headers(false);
@@ -365,8 +392,10 @@ curl_progress(void *unused, double dl_total, double dl_now,
 	      double ul_total, double ul_now)
 {
 	int ret = 0;
+// ^c will interrupt an http or ftp download.
+// Perhaps that is hanging, and blocking edbrowse.
 	if (intFlag) {
-		intFlag = false;
+		i_puts(MSG_Interrupted);
 		ret = 1;
 	}
 	return ret;
@@ -498,6 +527,8 @@ unpackUploadedFile(const char *post, const char *boundary,
 }				/* unpackUploadedFile */
 
 /* Pull a keyword: attribute out of an internet header. */
+/* Seems like nobody uses this. */
+#if 0
 static char *extractHeaderItem(const char *head, const char *end,
 			       const char *item, const char **ptr)
 {
@@ -525,6 +556,7 @@ static char *extractHeaderItem(const char *head, const char *end,
 	}
 	return h;
 }				/* extractHeaderItem */
+#endif
 
 /* This is a global function; it is called from cookies.c */
 char *extractHeaderParam(const char *str, const char *item)
@@ -533,7 +565,7 @@ char *extractHeaderParam(const char *str, const char *item)
 	const char *s = str;
 /* ; denotes the next param */
 /* Even the first param has to be preceeded by ; */
-	while (s = strchr(s, ';')) {
+	while ((s = strchr(s, ';'))) {
 		while (*s && (*s == ';' || (uchar) * s <= ' '))
 			s++;
 		if (!memEqualCI(s, item, le))
@@ -553,6 +585,7 @@ char *extractHeaderParam(const char *str, const char *item)
 
 /* Date format is:    Mon, 03 Jan 2000 21:29:33 GMT|[+-]nnnn */
 			/* Or perhaps:     Sun Nov  6 08:49:37 1994 */
+/* or perhaps 1994-11-06 08:49:37.nnnnZ */
 time_t parseHeaderDate(const char *date)
 {
 	static const char *const months[12] = {
@@ -562,20 +595,31 @@ time_t parseHeaderDate(const char *date)
 	time_t t = 0;
 	int zone = 0;
 	time_t now = 0;
-	time_t utcnow = 0;
 	int y;			/* remember the type of format */
 	struct tm *temptm = NULL;
 	struct tm tm;
+	long utcoffset = 0;
 	memset(&tm, 0, sizeof(struct tm));
 	tm.tm_isdst = -1;
+	const char *date0 = date;	// remember for debugging
 
 	now = time(NULL);
-	temptm = gmtime(&now);
+	temptm = localtime(&now);
 	if (temptm == NULL)
 		goto fail;
-	utcnow = mktime(temptm);
-	if (utcnow == -1 && errno)
-		goto fail;
+#ifndef _MSC_VER
+	utcoffset = temptm->tm_gmtoff;
+#endif
+
+	if (isdigitByte(date[0]) && isdigitByte(date[1]) &&
+	    isdigitByte(date[2]) && isdigitByte(date[3]) && date[4] == '-') {
+		y = 2;
+		tm.tm_year = atoi(date + 0) - 1900;
+		tm.tm_mon = atoi(date + 5) - 1;
+		tm.tm_mday = atoi(date + 8);
+		date += 11;
+		goto f3;
+	}
 
 /* skip past day of the week */
 	date = strchr(date, ' ');
@@ -603,13 +647,8 @@ f1:
 		date += 3;
 		if (*date == ' ') {
 			date++;
-			if (!isdigitByte(date[0]))
-				goto fail;
-			if (!isdigitByte(date[1]))
-				goto fail;
-			if (!isdigitByte(date[2]))
-				goto fail;
-			if (!isdigitByte(date[3]))
+			if (!isdigitByte(date[0]) || !isdigitByte(date[1]) ||
+			    !isdigitByte(date[2]) || !isdigitByte(date[3]))
 				goto fail;
 			tm.tm_year =
 			    (date[0] - '0') * 1000 + (date[1] - '0') * 100 +
@@ -618,9 +657,7 @@ f1:
 		} else if (*date == '-') {
 			/* Sunday, 06-Nov-94 08:49:37 GMT */
 			date++;
-			if (!isdigitByte(date[0]))
-				goto fail;
-			if (!isdigitByte(date[1]))
+			if (!isdigitByte(date[0]) || !isdigitByte(date[1]))
 				goto fail;
 			if (!isdigitByte(date[2])) {
 				tm.tm_year =
@@ -663,44 +700,36 @@ f2:
 		date++;
 	}
 
+f3:
 /* ready to crack time */
-	if (!isdigitByte(date[0]))
-		goto fail;
-	if (!isdigitByte(date[1]))
+	if (!isdigitByte(date[0]) || !isdigitByte(date[1]))
 		goto fail;
 	tm.tm_hour = (date[0] - '0') * 10 + date[1] - '0';
 	date += 2;
 	if (*date != ':')
 		goto fail;
 	date++;
-	if (!isdigitByte(date[0]))
-		goto fail;
-	if (!isdigitByte(date[1]))
+	if (!isdigitByte(date[0]) || !isdigitByte(date[1]))
 		goto fail;
 	tm.tm_min = (date[0] - '0') * 10 + date[1] - '0';
 	date += 2;
 	if (*date != ':')
 		goto fail;
 	date++;
-	if (!isdigitByte(date[0]))
-		goto fail;
-	if (!isdigitByte(date[1]))
+	if (!isdigitByte(date[0]) || !isdigitByte(date[1]))
 		goto fail;
 	tm.tm_sec = (date[0] - '0') * 10 + date[1] - '0';
 	date += 2;
+	if (y == 2)
+		goto f4;
 
-	if (y) {
+	if (y == 1) {
 /* year is at the end */
 		if (*date != ' ')
 			goto fail;
 		date++;
-		if (!isdigitByte(date[0]))
-			goto fail;
-		if (!isdigitByte(date[1]))
-			goto fail;
-		if (!isdigitByte(date[2]))
-			goto fail;
-		if (!isdigitByte(date[3]))
+		if (!isdigitByte(date[0]) || !isdigitByte(date[1]) ||
+		    !isdigitByte(date[2]) || !isdigitByte(date[3]))
 			goto fail;
 		tm.tm_year =
 		    (date[0] - '0') * 1000 + (date[1] - '0') * 100 + (date[2] -
@@ -726,11 +755,13 @@ f2:
 			zone = -zone;
 	}
 
+f4:
 	t = mktime(&tm);
 	if (t != (time_t) - 1)
-		return t + zone + (now - utcnow);
+		return t + zone + utcoffset;
 
 fail:
+	debugPrint(3, "parseHeaderDate fails on %s", date0);
 	return 0;
 }				/* parseHeaderDate */
 
@@ -895,6 +926,14 @@ bool httpConnect(const char *url, bool down_ok, bool webpage,
 		return false;
 	}
 
+	if (!curlActive) {
+		eb_curl_global_init();
+		cookiesFromJar();
+		setupEdbrowseCache();
+	}
+
+	cf->fromcurl = true;
+
 	if (stringEqualCI(prot, "http") || stringEqualCI(prot, "https")) {
 		;		/* ok for now */
 	} else if (stringEqualCI(prot, "ftp") ||
@@ -1008,7 +1047,7 @@ mimestream:
 			goto curl_fail;
 	}
 
-	if (sendReferrer && currentReferrer) {
+	if (sendReferrer && isURL(currentReferrer)) {
 		const char *post2 = strchr(currentReferrer, '\1');
 		if (!post2)
 			post2 = currentReferrer + strlen(currentReferrer);
@@ -1092,6 +1131,31 @@ perform:
 		ht_cacheable = true;
 		curlret = fetch_internet(h);
 
+/*********************************************************************
+This is a one line workaround for an apparent bug in curl.
+The return CURLE_WRITE_ERROR means the data fetched from the internet
+could not be written to disk. And how does curl know?
+Because the callback function returns a lesser number of bytes.
+This is like write(), if it returns a lesser number
+of bytes then it was unable to write the entire block to disk.
+Ok, but I never return fewer bytes than was passed to me.
+I return the expected number of bytes, or -1 in the rare case
+that I want to abort the download.
+So you see, curl should never return this WRITE error.
+Yet it does, in version 7.58.0-2, on debian.
+And only on one page we have found so far:
+https://www.literotica.com/stories/new_submissions.php
+The entire page is downloaded, down to the very last byte,
+then the WRITE error is passed back.
+Well if it happens once it will happen elsewhere.
+Users will not be able to fetch pages from the internet, and not know why.
+The error message, can't write to disk, is not helpful at all.
+So this is a simple workaround.
+*********************************************************************/
+
+		if (curlret == CURLE_WRITE_ERROR)
+			curlret = CURLE_OK;
+
 		if (cbd.down_state == 6) {
 			curl_easy_cleanup(h);
 			goto mimestream;
@@ -1187,8 +1251,8 @@ they go where they go, so this doesn't come up very often.
 		}
 
 		if (allowRedirection &&
-		    (ht_code >= 301 && ht_code <= 303 ||
-		     ht_code >= 307 && ht_code <= 308)) {
+		    ((ht_code >= 301 && ht_code <= 303) ||
+		     (ht_code >= 307 && ht_code <= 308))) {
 			redir = newlocation;
 			if (redir)
 				redir = resolveURL(urlcopy, redir);
@@ -1244,12 +1308,23 @@ they go where they go, so this doesn't come up very often.
 		}
 
 		else if (ht_code == 401 && !proceed_unauthenticated) {
-			bool got_creds;
-			i_printf(MSG_AuthRequired, urlcopy);
-			nl();
-			got_creds = read_credentials(creds_buf);
+			bool got_creds = false;
+
+			/* only try realm on first try - prevents loop */
+			if (strcmp(creds_buf, ":") == 0)
+				got_creds =
+				    getUserPassRealm(urlcopy, creds_buf,
+						     ht_auth_realm);
+
+			if (!got_creds) {
+				i_printf(MSG_AuthRequired, urlcopy,
+					 ht_auth_realm);
+				nl();
+				got_creds = read_credentials(creds_buf);
+			}
 			if (got_creds) {
-				addWebAuthorization(urlcopy, creds_buf, false);
+				addWebAuthorization(urlcopy, creds_buf, false,
+						    ht_auth_realm);
 				curl_easy_setopt(h, CURLOPT_USERPWD, creds_buf);
 				nzFree(serverData);
 				serverData = emptyString;
@@ -1305,9 +1380,9 @@ curl_fail:
 		serverDataLen = 0;
 		nzFree(urlcopy);	/* Free it on transfer failure. */
 	} else {
-		if (ht_code != 200 && ht_code != 201 &&
-		    (webpage || debugLevel >= 2) ||
-		    ht_code == 201 && debugLevel >= 3)
+		if ((ht_code != 200 && ht_code != 201 &&
+		     (webpage || debugLevel >= 2)) ||
+		    (ht_code == 201 && debugLevel >= 3))
 			i_printf(MSG_HTTPError,
 				 ht_code, message_for_response_code(ht_code));
 		if (name_changed)
@@ -1362,7 +1437,7 @@ static void ftpls(char *line)
 		line[--l] = 0;
 
 /* blank line becomes paragraph break */
-	if (!l || memEqualCI(line, "total ", 6) && stringIsNum(line + 6)) {
+	if (!l || (memEqualCI(line, "total ", 6) && stringIsNum(line + 6))) {
 		stringAndString(&serverData, &serverDataLen, "<P>\n");
 		return;
 	}
@@ -1432,7 +1507,7 @@ static void ftpls(char *line)
 		stringAndString(&serverData, &serverDataLen, line);
 	} else {
 		char c, *q;
-		for (q = line; c = *q; ++q) {
+		for (q = line; (c = *q); ++q) {
 			char *meta = 0;
 			if (c == '<')
 				meta = "&lt;";
@@ -1532,7 +1607,10 @@ void ebcurl_setError(CURLcode curlret, const char *url)
 		break;
 
 	case CURLE_ABORTED_BY_CALLBACK:
+#if 0
+// this is printed by the callback function
 		setError(MSG_Interrupted);
+#endif
 		break;
 /* These all look like session initiation failures. */
 	case CURLE_FTP_WEIRD_SERVER_REPLY:
@@ -1758,6 +1836,7 @@ my_curl_safeSocket(void *clientp, curl_socket_t socketfd, curlsocktype purpose)
 static CURL *http_curl_init(struct eb_curl_callback_data *cbd)
 {
 	CURLcode curl_init_status = CURLE_OK;
+	int curl_auth;
 	CURL *h = curl_easy_init();
 	if (h == NULL)
 		goto libcurl_init_fail;
@@ -1793,12 +1872,18 @@ static CURL *http_curl_init(struct eb_curl_callback_data *cbd)
 
 /*
 * tell libcurl to pick the strongest method from basic, digest and ntlm authentication
-* don't use any auth method as it will prefer Negotiate to NTLM,
+* don't use any auth method by default as it will prefer Negotiate to NTLM,
 * and it looks like in most cases microsoft IIS says it supports both and libcurl
 * doesn't fall back to NTLM when it discovers that Negotiate isn't set up on a system
 */
-	curl_easy_setopt(h, CURLOPT_HTTPAUTH,
-			 CURLAUTH_BASIC | CURLAUTH_DIGEST | CURLAUTH_NTLM);
+	curl_auth = CURLAUTH_BASIC | CURLAUTH_DIGEST | CURLAUTH_NTLM;
+	if (curlAuthNegotiate)
+#ifdef CURLAUTH_NEGOTIATE
+		curl_auth |= CURLAUTH_NEGOTIATE;
+#else
+		curl_auth |= CURLAUTH_GSSNEGOTIATE;	/* libcurl < 7.38 */
+#endif
+	curl_easy_setopt(h, CURLOPT_HTTPAUTH, curl_auth);
 
 /* The next few setopt calls could allocate or perform file I/O. */
 	ht_error[0] = '\0';
@@ -1917,7 +2002,8 @@ static const char *message_for_response_code(int code)
   ** prompt: prompt that user should see.
   ** buffer: buffer into which the data should be stored.
   ** max_length: maximum allowable length of input.
- ** error_msg: message to display if input exceeds maximum length.
+  ** error_msg: message to display if input exceeds maximum length.
+  ** hide_echo: whether to disable terminal echo (sensitive input)
  * Note: prompt and error_message should be message constants from messages.h.
  * Return value: none.  buffer contains input on return. */
 
@@ -1926,15 +2012,23 @@ static const char *message_for_response_code(int code)
  * in a function, and we call that function twice.
  * After the call, the buffer contains the user's input, without a newline.
  * The return value is the length of the string in buffer. */
-static int
-prompt_and_read(int prompt, char *buffer, int buffer_length, int error_message)
+int
+prompt_and_read(int prompt, char *buffer, int buffer_length, int error_message,
+		bool hide_echo)
 {
 	bool reading = true;
 	int n = 0;
+
 	while (reading) {
+		char *s;
+		if (hide_echo)
+			ttySetEcho(false);
 		i_printf(prompt);
 		fflush(stdout);
-		if (!fgets(buffer, buffer_length, stdin))
+		s = fgets(buffer, buffer_length, stdin);
+		if (hide_echo)
+			ttySetEcho(true);
+		if (!s)
 			ebClose(0);
 		n = strlen(buffer);
 		if (n && buffer[n - 1] == '\n')
@@ -1974,11 +2068,11 @@ static bool read_credentials(char *buffer)
 		i_puts(MSG_WebAuthorize);
 		input_length =
 		    prompt_and_read(MSG_UserName, buffer, MAXUSERPASS,
-				    MSG_UserNameLong);
+				    MSG_UserNameLong, false);
 		if (!stringEqual(buffer, "x")) {
 			char *password_ptr = buffer + input_length + 1;
 			prompt_and_read(MSG_Password, password_ptr, MAXUSERPASS,
-					MSG_PasswordLong);
+					MSG_PasswordLong, true);
 			if (!stringEqual(password_ptr, "x")) {
 				got_creds = true;
 				*(password_ptr - 1) = ':';	/* separate user and password with colon. */
@@ -2174,8 +2268,6 @@ static void background_download(struct eb_curl_callback_data *data)
 		return;
 	}
 
-/* child doesn't need javascript */
-	js_disconnect();
 /* ignore interrupt, not sure about quit and hangup */
 	signal(SIGINT, SIG_IGN);
 	data->down_state = 4;
@@ -2206,7 +2298,6 @@ int bg_jobs(bool iponly)
 /* in progress */
 	part = false;
 	foreach(j, down_jobs) {
-		size_t now_size;
 		if (j->state != 4)
 			continue;
 		++numback;
@@ -2217,7 +2308,7 @@ int bg_jobs(bool iponly)
 		}
 		printf("%s", j->file + j->file2);
 		if (j->fsize)
-			printf(" %d/%lu",
+			printf(" %d/%zu",
 			       (fileSizeByName(j->file) / CHUNKSIZE), j->fsize);
 		nl();
 	}
@@ -2397,7 +2488,6 @@ const char *findProxyForURL(const char *url)
  * Pass a range of lines; you can expand all the frames in one go.
  * Return false if there is a problem fetching a web page,
  * or if none of the lines are frames. */
-static int frameExpandLine(int lineNumber);
 static int frameContractLine(int lineNumber);
 static const char *stringInBufLine(const char *s, const char *t);
 bool frameExpand(bool expand, int ln1, int ln2)
@@ -2408,7 +2498,7 @@ bool frameExpand(bool expand, int ln1, int ln2)
 
 	for (ln = ln1; ln <= ln2; ++ln) {
 		if (expand)
-			p = frameExpandLine(ln);
+			p = frameExpandLine(ln, NULL);
 		else
 			p = frameContractLine(ln);
 		if (p > problem)
@@ -2430,24 +2520,32 @@ bool frameExpand(bool expand, int ln1, int ln2)
  1 line is not a frame.
  2 frame doesn't have a valid url.
  3 Problem fetching the rul or rendering the page.  */
-static int frameExpandLine(int ln)
+int frameExpandLine(int ln, jsobjtype fo)
 {
 	pst line;
 	int tagno, start;
 	const char *s;
 	struct htmlTag *t;
-	struct ebFrame *save_cf, *last_f;
+	struct ebFrame *save_cf, *new_cf, *last_f;
+	uchar save_local;
+	struct htmlTag *cdt;	// contentDocument tag
 
-	line = fetchLine(ln, -1);
-	s = stringInBufLine(line, "Frame ");
-	if (!s)
-		return 1;
-	if ((s = strchr(s, InternalCodeChar)) == NULL)
-		return 2;
-	tagno = strtol(s + 1, (char **)&s, 10);
-	if (tagno < 0 || tagno >= cw->numTags || *s != '{')
-		return 2;
-	t = tagList[tagno];
+	if (fo) {
+		t = tagFromJavaVar(fo);
+		if (!t)
+			return 1;
+	} else {
+		line = fetchLine(ln, -1);
+		s = stringInBufLine((char *)line, "Frame ");
+		if (!s)
+			return 1;
+		if ((s = strchr(s, InternalCodeChar)) == NULL)
+			return 2;
+		tagno = strtol(s + 1, (char **)&s, 10);
+		if (tagno < 0 || tagno >= cw->numTags || *s != '{')
+			return 2;
+		t = tagList[tagno];
+	}
 	if (t->action != TAGACT_FRAME)
 		return 1;
 
@@ -2458,26 +2556,40 @@ static int frameExpandLine(int ln)
 	}
 
 	s = t->href;
-	if (!s)
-		return 2;
+	if (!s) {
+// No source. If this is your request then return an error.
+// But if we're dipping into the objects then it needs to expand
+// into a separate window, a separate js space, with an empty body.
+		if (!fo)
+			return 2;
+// After expansion we need to be able to expand it,
+// because there's something there, well maybe.
+		t->href = cloneString("#");
+	}
 
-	save_cf = cf;
+	save_cf = cf = t->f0;
 /* have to push a new frame before we read the web page */
 	for (last_f = &(cw->f0); last_f->next; last_f = last_f->next) ;
 	last_f->next = cf = allocZeroMem(sizeof(struct ebFrame));
 	cf->owner = cw;
 	cf->frametag = t;
-	debugPrint(2, "fetch frame %s", s);
-	if (!readFileArgv(s)) {
+	debugPrint(2, "fetch frame %s", (s ? s : "empty"));
+	if (s) {
+		bool rc;
+		currentReferrer = cloneString(save_cf->fileName);
+		rc = readFileArgv(s);
+		nzFree(currentReferrer);
+		currentReferrer = NULL;
+		if (!rc) {
 /* serverData was never set, or was freed do to some other error. */
 /* We just need to pop the frame and return. */
-		fileSize = -1;	/* don't print 0 */
-		nzFree(cf->fileName);
-		free(cf);
-		last_f->next = 0;
-		cf = save_cf;
-		return 3;
-	}
+			fileSize = -1;	/* don't print 0 */
+			nzFree(cf->fileName);
+			free(cf);
+			last_f->next = 0;
+			cf = save_cf;
+			return 3;
+		}
 
 /*********************************************************************
 readFile could return success and yet serverData is null.
@@ -2487,15 +2599,20 @@ It can, if the frame is a youtube video, which is not unusual at all.
 So check for serverData null here. Once again we pop the frame.
 *********************************************************************/
 
-	if (serverData == NULL) {
-		nzFree(cf->fileName);
-		free(cf);
-		last_f->next = 0;
-		cf = save_cf;
-		fileSize = -1;
-		return 0;
+		if (serverData == NULL) {
+			nzFree(cf->fileName);
+			free(cf);
+			last_f->next = 0;
+			cf = save_cf;
+			fileSize = -1;
+			return 0;
+		}
+	} else {
+		serverData = cloneString("<body></body>");
+		serverDataLen = strlen(serverData);
 	}
 
+	new_cf = cf;
 	if (changeFileName) {
 		nzFree(cf->fileName);
 		cf->fileName = changeFileName;
@@ -2513,6 +2630,7 @@ So check for serverData null here. Once again we pop the frame.
  * but I'm too lazy to do that right now, so I'll just assume it's good. */
 
 	cf->hbase = cloneString(cf->fileName);
+	save_local = browseLocal;
 	browseLocal = !isURL(cf->fileName);
 	prepareForBrowse(serverData, serverDataLen);
 	if (javaOK(cf->fileName))
@@ -2524,15 +2642,31 @@ So check for serverData null here. Once again we pop the frame.
 /* call the tidy parser to build the html nodes */
 	html2nodes(serverData, true);
 	nzFree(serverData);	/* don't need it any more */
+	serverData = 0;
 	htmlGenerated = false;
-	htmlNodesIntoTree(start, t);
+// in the edbrowse world, the only child of the frame tag
+// is the contentDocument tag.
+	cdt = t->firstchild;
+	htmlNodesIntoTree(start, cdt);
+	cdt->step = 0;
 	prerender(0);
 	if (isJSAlive) {
+		jsobjtype topobj;
 		decorate(0);
 		set_basehref(cf->hbase);
+// parent points to the containing frame.
+		set_property_object(cf->winobj, "parent", save_cf->winobj);
+// And top points to the top.
+		cf = save_cf;
+		topobj = get_property_object(cf->winobj, "top");
+		cf = new_cf;
+		set_property_object(cf->winobj, "top", topobj);
+		set_property_object(cf->winobj, "frameElement", t->jv);
+		run_function_bool(cf->winobj, "eb$qs$start");
 		runScriptsPending();
 		runOnload();
 		runScriptsPending();
+		rebuildSelectors();
 		set_property_string(cf->docobj, "readyState", "complete");
 	}
 
@@ -2544,6 +2678,26 @@ So check for serverData null here. Once again we pop the frame.
 
 	t->f1 = cf;
 	cf = save_cf;
+	browseLocal = save_local;
+	if (fo)
+		t->contracted = true;
+	if (isJSAlive) {
+		jsobjtype cdo;	// contentDocument object
+		jsobjtype cwo;	// contentWindow object
+		jsobjtype cna;	// childNodes array
+		cdo = new_cf->docobj;
+		disconnectTagObject(cdt);
+		connectTagObject(cdt, cdo);
+		set_property_object(t->jv, "content$Document", cdo);
+		cna = get_property_object(t->jv, "childNodes");
+		set_array_element_object(cna, 0, cdo);
+		cwo = new_cf->winobj;
+		set_property_object(t->jv, "content$Window", cwo);
+// run the frame unload function if it is there.
+// I assume it should run in the higher context.
+		run_event_bool(t->jv, t->info->name, "onload");
+	}
+
 	return 0;
 }				/* frameExpandLine */
 
@@ -2601,20 +2755,23 @@ static const char *stringInBufLine(const char *s, const char *t)
 bool reexpandFrame(void)
 {
 	int j, start;
-	struct htmlTag *t, *frametag;
+	struct htmlTag *frametag;
+	struct htmlTag *cdt;	// contentDocument tag
+	uchar save_local;
+	bool rc;
+	jsobjtype save_top, save_parent, save_fe;
 
-/* cut the children off from the frame tag */
 	cf = newloc_f;
 	frametag = cf->frametag;
-	for (t = frametag->firstchild; t; t = t->sibling) {
-		t->deleted = true;
-		t->step = 100;
-		t->parent = 0;
-	}
-	frametag->firstchild = 0;
+	cdt = frametag->firstchild;
+	save_top = get_property_object(cf->winobj, "top");
+	save_parent = get_property_object(cf->winobj, "parent");
+	save_fe = get_property_object(cf->winobj, "frameElement");
+
+// Cut away objects from the previous document, which are now inaccessible.
+	underKill(cdt);
 
 	delTimers(cf);
-	delInputChanges(cf);
 	freeJavaContext(cf);
 	nzFree(cf->dw);
 	cf->dw = 0;
@@ -2626,8 +2783,12 @@ bool reexpandFrame(void)
 	cf->f_encoded = false;
 	nzFree(cf->firstURL);
 	cf->firstURL = 0;
+	currentReferrer = cloneString(cf->fileName);
 
-	if (!readFileArgv(cf->fileName)) {
+	rc = readFileArgv(cf->fileName);
+	nzFree(currentReferrer);
+	currentReferrer = NULL;
+	if (!rc) {
 /* serverData was never set, or was freed do to some other error. */
 		fileSize = -1;	/* don't print 0 */
 		return false;
@@ -2650,29 +2811,72 @@ bool reexpandFrame(void)
 	fileSize = -1;
 
 	cf->hbase = cloneString(cf->fileName);
+	save_local = browseLocal;
 	browseLocal = !isURL(cf->fileName);
 	prepareForBrowse(serverData, serverDataLen);
 	if (javaOK(cf->fileName))
 		createJavaContext();
+
 	start = cw->numTags;
 /* call the tidy parser to build the html nodes */
 	html2nodes(serverData, true);
 	nzFree(serverData);	/* don't need it any more */
+	serverData = 0;
 	htmlGenerated = false;
-	htmlNodesIntoTree(start, frametag);
+	htmlNodesIntoTree(start, cdt);
+	cdt->step = 0;
 	prerender(0);
 	if (isJSAlive) {
 		decorate(0);
 		set_basehref(cf->hbase);
+		set_property_object(cf->winobj, "top", save_top);
+		set_property_object(cf->winobj, "parent", save_parent);
+		set_property_object(cf->winobj, "frameElement", save_fe);
+		run_function_bool(cf->winobj, "eb$qs$start");
 		runScriptsPending();
 		runOnload();
 		runScriptsPending();
+		rebuildSelectors();
 		set_property_string(cf->docobj, "readyState", "complete");
 	}
 
 	j = strlen(cf->fileName);
 	cf->fileName = reallocMem(cf->fileName, j + 8);
 	strcat(cf->fileName, ".browse");
+	browseLocal = save_local;
+
+	if (isJSAlive) {
+		struct ebFrame *save_cf;
+		jsobjtype cdo;	// contentDocument object
+		jsobjtype cwo;	// contentWindow object
+		jsobjtype cna;	// childNodes array
+		cdo = cf->docobj;
+		cwo = cf->winobj;
+		disconnectTagObject(cdt);
+		connectTagObject(cdt, cdo);
+// have to point contentDocument to the new document object,
+// but that requires a change of context.
+		save_cf = cf;
+		cf = frametag->f0;
+		set_property_object(frametag->jv, "content$Document", cdo);
+		cna = get_property_object(frametag->jv, "childNodes");
+		set_array_element_object(cna, 0, cdo);
+		set_property_object(frametag->jv, "content$Window", cwo);
+		cf = save_cf;
+	}
 
 	return true;
 }				/* reexpandFrame */
+
+// Make sure a web page is not trying to read a local file.
+bool frameSecurityFile(const char *thisfile)
+{
+	struct ebFrame *f = &cf->owner->f0;
+	for (; f != cf; f = f->next) {
+		if (!f->fromcurl)
+			continue;
+		setError(MSG_NoAccessSecure, thisfile);
+		return false;
+	}
+	return true;
+}
