@@ -5,7 +5,6 @@
 
 #include "eb.h"
 
-#include <sys/stat.h>
 #include <pthread.h>
 #include <pcre.h>
 
@@ -14,7 +13,7 @@
 
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.6.3";
+const char *version = "3.7.2";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
@@ -22,12 +21,11 @@ char *recycleBin, *sigFile, *sigFileEnd;
 char *cacheDir;
 int cacheSize = 1000, cacheCount = 10000;
 char *ebTempDir, *ebUserDir;
-char *userAgents[10];
+char *userAgents[MAXAGENT + 1];
 char *currentAgent, *currentReferrer;
 bool allowRedirection = true, allowJS = true, sendReferrer = true;
 bool allowXHR = true;
 bool ftpActive;
-int jsPool = 32;
 int webTimeout = 20, mailTimeout = 0;
 int displayLength = 500;
 int verifyCertificates = 1;
@@ -59,6 +57,7 @@ static int javaDisCount;
 static struct DBTABLE dbtables[MAXDBT];
 static int numTables;
 volatile bool intFlag;
+bool curlActive;
 bool ismc, isimap, passMail;
 char whichproc = 'e';		// edbrowse
 bool inInput, listNA;
@@ -68,9 +67,11 @@ bool fetchBlobColumns;
 bool caseInsensitive, searchStringsAll;
 bool binaryDetect = true;
 bool inputReadLine;
+bool curlAuthNegotiate = false;
 int context = 1;
 pst linePending;
 struct ebSession sessionList[MAXSESSION], *cs;
+int maxSession;
 static pthread_mutex_t share_mutex = PTHREAD_MUTEX_INITIALIZER;
 
 #define MAXNOJS 500
@@ -100,8 +101,7 @@ const char *mailRedirect(const char *to, const char *from,
 
 	for (f = mailFilters; f->match; ++f) {
 		const char *m = f->match;
-		int mlen = strlen(m);
-		int j, k;
+		int k, mlen = strlen(m);
 
 		r = f->redirect;
 
@@ -211,9 +211,8 @@ static void catchSig(int n)
 
 bool isSQL(const char *s)
 {
-	char c;
-	const char *c1 = 0, *c2 = 0;
-	c = *s;
+	char c = *s;
+	const char *c1 = 0;
 
 	if (!sqlPresent)
 		goto no;
@@ -221,10 +220,11 @@ bool isSQL(const char *s)
 	if (isURL(s))
 		goto no;
 
+// look for word] or word:word]
 	if (!isalphaByte(c))
 		goto no;
 
-	for (++s; c = *s; ++s) {
+	for (++s; (c = *s); ++s) {
 		if (c == '_')
 			continue;
 		if (isalnumByte(c))
@@ -235,10 +235,8 @@ bool isSQL(const char *s)
 			c1 = s;
 			continue;
 		}
-		if (c == ']') {
-			c2 = s;
+		if (c == ']')
 			goto yes;
-		}
 	}
 
 no:
@@ -326,7 +324,7 @@ void eb_curl_global_init(void)
 		if (curl_init_status != CURLE_OK)
 			goto libcurl_init_fail;
 	}
-	if (cookieFile && whichproc == 'e') {
+	if (cookieFile && !ismc) {
 		curl_init_status =
 		    curl_easy_setopt(global_http_handle, CURLOPT_COOKIEJAR,
 				     cookieFile);
@@ -343,6 +341,7 @@ void eb_curl_global_init(void)
 			     global_share_handle);
 	if (curl_init_status != CURLE_OK)
 		goto libcurl_init_fail;
+	curlActive = true;
 	return;
 
 libcurl_init_fail:
@@ -359,8 +358,10 @@ void ebClose(int n)
 {
 	bg_jobs(true);
 	dbClose();
-	js_shutdown();
-	eb_curl_global_cleanup();
+	if (curlActive) {
+		mergeCookies();
+		eb_curl_global_cleanup();
+	}
 	exit(n);
 }				/* ebClose */
 
@@ -394,8 +395,7 @@ static void setupEdbrowseTempDirectory(void)
 /* no such directory, try to make it */
 /* this temp edbrowse directory is used by everyone system wide */
 		if (mkdir(ebTempDir, 0777)) {
-			if (whichproc == 'e')
-				i_printf(MSG_TempDir, ebTempDir);
+			i_printf(MSG_TempDir, ebTempDir);
 			ebTempDir = 0;
 			return;
 		}
@@ -408,8 +408,7 @@ static void setupEdbrowseTempDirectory(void)
 	if (fileTypeByName(ebUserDir, false) != 'd') {
 /* no such directory, try to make it */
 		if (mkdir(ebUserDir, 0700)) {
-			if (whichproc == 'e')
-				i_printf(MSG_TempDir, ebUserDir);
+			i_printf(MSG_TempDir, ebUserDir);
 			ebUserDir = 0;
 			return;
 		}
@@ -442,6 +441,7 @@ int main(int argc, char **argv)
 #endif // !_MSC_VER
 
 	selectLanguage();
+	setHTTPLanguage(eb_language);
 
 /* Establish the home directory, and standard edbrowse files thereunder. */
 	home = getenv("HOME");
@@ -529,25 +529,6 @@ int main(int argc, char **argv)
 	progname = argv[0];
 	++argv, --argc;
 
-// look for --mode on the arg list.
-	if (stringEqual(argv[0], "--mode")) {
-		char *m;
-		if (argc == 1)
-			i_printfExit(MSG_Usage);
-		m = argv[1];
-		argv += 2;
-		argc -= 2;
-		if (stringEqual(m, "js"))
-			whichproc = 'j';
-		else if (stringEqual(m, "curl"))
-			whichproc = 'c';
-		else
-			i_printfExit(MSG_Usage);
-	}
-
-	if (whichproc == 'j')
-		return js_main(argc, argv);
-
 	ttySaveSettings();
 	initializeReadline();
 
@@ -566,12 +547,6 @@ int main(int argc, char **argv)
 	}
 	account = localAccount;
 
-// With config file read, we can now set up the global http handle
-// with certificate file and cookie jar etc.
-	eb_curl_global_init();
-	cookiesFromJar();
-	setupEdbrowseCache();
-
 	for (; argc && argv[0][0] == '-'; ++argv, --argc) {
 		char *s = *argv;
 		++s;
@@ -620,6 +595,7 @@ int main(int argc, char **argv)
 			if (!*s) {
 				ismc = true;	/* running as a mail client */
 				allowJS = false;	/* no javascript in mail client */
+				eb_curl_global_init();
 				++argv, --argc;
 				if (!argc || !dofetch)
 					break;
@@ -712,6 +688,11 @@ int main(int argc, char **argv)
 		cxSwitch(cx, false);
 		if (cx == 1)
 			runEbFunction("init");
+		if (file[0] == '<') {
+			runEbFunction(file + 1);
+			++argv, --argc;
+			continue;
+		}
 		changeFileName = 0;
 		cf->fileName = cloneString(file);
 		cf->firstURL = cloneString(file);
@@ -731,9 +712,10 @@ int main(int argc, char **argv)
 
 		cw->undoable = cw->changeMode = false;
 /* Browse the text if it's a url */
-		if (rc && isURL(cf->fileName) &&
-		    (cf->mt && cf->mt->outtype
-		     || isBrowseableURL(cf->fileName))) {
+		if (rc && isURL(cf->fileName) && ((cf->mt && cf->mt->outtype)
+						  ||
+						  isBrowseableURL
+						  (cf->fileName))) {
 			if (runCommand("b"))
 				debugPrint(1, "%d", fileSize);
 			else
@@ -871,7 +853,7 @@ bool runEbFunction(const char *line)
 	fncopy = cloneString(ip);
 	ip = fncopy;
 
-	while (code = *ip) {
+	while ((code = *ip)) {
 		if (intFlag) {
 			setError(MSG_Interrupted);
 			goto fail;
@@ -1052,10 +1034,9 @@ void unreadConfigFile(void)
 
 	webTimeout = mailTimeout = 0;
 	displayLength = 500;
-	jsPool = 32;
 
 	setDataSource(NULL);
-	setHTTPLanguage(NULL);
+	setHTTPLanguage(eb_language);
 	deleteNovsHosts();
 }				/* unreadConfigFile */
 
@@ -1074,7 +1055,7 @@ static const char *const keywords[] = {
 	"downdir", "maildir", "agent",
 	"jar", "nojs", "cachedir",
 	"webtimer", "mailtimer", "certfile", "datasource", "proxy",
-	"linelength", "localizeweb", "jspool", "novs", "cachesize",
+	"linelength", "localizeweb", "notused33", "novs", "cachesize",
 	"adbook", 0
 };
 
@@ -1463,9 +1444,9 @@ putc:
 		case 19:	/* keycol */
 			if (!isdigitByte(*v))
 				cfgLine0(MSG_EBRC_KeyNotNb);
-			td->key1 = strtol(v, &v, 10);
+			td->key1 = (uchar) strtol(v, &v, 10);
 			if (*v == ',' && isdigitByte(v[1]))
-				td->key2 = strtol(v + 1, &v, 10);
+				td->key2 = (uchar) strtol(v + 1, &v, 10);
 			if (td->key1 > td->ncols || td->key2 > td->ncols)
 				cfgLine1(MSG_EBRC_KeyOutRange, td->ncols);
 			continue;
@@ -1493,11 +1474,11 @@ putc:
 			continue;
 
 		case 22:	/* agent */
-			for (j = 0; j < 10; ++j)
+			for (j = 0; j < MAXAGENT; ++j)
 				if (!userAgents[j])
 					break;
-			if (j == 10)
-				cfgLine0(MSG_EBRC_ManyAgents);
+			if (j == MAXAGENT)
+				cfgLine1(MSG_EBRC_ManyAgents, MAXAGENT);
 			userAgents[j] = v;
 			continue;
 
@@ -1587,14 +1568,6 @@ putc:
 			setHTTPLanguage(v);
 			continue;
 
-		case 33:	/* jspool */
-			jsPool = atoi(v);
-			if (jsPool < 2)
-				jsPool = 2;
-			if (jsPool > 1000)
-				jsPool = 1000;
-			continue;
-
 		case 34:	/* novs */
 			if (*v == '.')
 				++v;
@@ -1676,13 +1649,15 @@ nokeyword:
 					cfgLine0(MSG_EBRC_NoReply);
 				if (act->secure)
 					act->inssl = act->outssl = 1;
-				if (!act->inport)
-					if (act->secure)
+				if (!act->inport) {
+					if (act->secure) {
 						act->inport =
 						    (act->imap ? 993 : 995);
-					else
+					} else {
 						act->inport =
 						    (act->imap ? 143 : 110);
+					}
+				}
 				if (!act->outport)
 					act->outport = (act->secure ? 465 : 25);
 				continue;
