@@ -62,7 +62,7 @@ void unpercentURL(char *url)
 	char c, *u, *w;
 	int n;
 	u = w = url;
-	while (c = *u) {
+	while ((c = *u)) {
 		++u;
 		if (c == '+')
 			c = ' ';
@@ -91,7 +91,7 @@ void unpercentString(char *s)
 {
 	char c, *u, *w;
 	u = w = s;
-	while (c = *u) {
+	while ((c = *u)) {
 		++u;
 		if (c == '+')
 			c = ' ';
@@ -370,7 +370,7 @@ static int parseURL(const char *url, const char **proto, int *prlen, const char
 	if (a < 0)
 		return false;
 
-	if (free_syntax = protocols[a].free_syntax) {
+	if ((free_syntax = protocols[a].free_syntax)) {
 		if (data)
 			*data = p;
 		if (dalen)
@@ -740,7 +740,7 @@ static void snipLastSegment(char **path, int *pathLen)
 static void squashDirectories(char *url)
 {
 	char *dd = (char *)getDataURL(url);
-	char *s, *t, *end;
+	char *s, *end;
 	char *inPath = NULL;
 	char *outPath;
 	int outPathLen = 0;
@@ -805,23 +805,12 @@ char *resolveURL(const char *base, const char *rel)
 	if (memEqualCI(rel, "data:", 5))
 		return cloneString(rel);
 
+	debugPrint(5, "resolve(%s|%s)", base, rel);
 	if (!base)
 		base = emptyString;
+	if (!rel)
+		rel = emptyString;
 	n = allocString(strlen(base) + strlen(rel) + 12);
-	debugPrint(5, "resolve(%s|%s)", base, rel);
-
-#if 0
-// This is specifically to work around a tidy bug.
-// fixed in 5.1.36 so not needed any more.
-	for (s = rel; *s == '%'; s += 3) {
-		if (s[1] == '2' && s[2] == '0')
-			continue;
-		break;
-	}
-	if (s > rel && memEqualCI(s, "http", 4))
-		rel = s;
-// end tidy workaround
-#endif
 
 	if (rel[0] == '#' && !strchr(rel, '/')) {
 /* This is an anchor for the current document, don't resolve. */
@@ -842,7 +831,7 @@ out_n:
 	}
 
 	if (rel[0] == '/' && rel[1] == '/') {
-		if (s = strstr(base, "//")) {
+		if ((s = strstr(base, "//"))) {
 			strncpy(n, base, s - base);
 			n[s - base] = 0;
 		} else
@@ -932,9 +921,9 @@ bool sameURL(const char *s, const char *t)
 		return false;
 
 /* lop off hash */
-	if (u = findHash(s))
+	if ((u = findHash(s)))
 		p = u;
-	if (u = findHash(t))
+	if ((u = findHash(t)))
 		q = u;
 
 /* It's ok if one says http and the other implies it. */
@@ -1027,7 +1016,7 @@ char *encodePostData(const char *s)
 	if (s == emptyString)
 		return emptyString;
 	post = initString(&l);
-	while (c = *s++) {
+	while ((c = *s++)) {
 		if (isalnumByte(c))
 			goto putc;
 		if (strchr("-._~*()!", c))
